@@ -1,5 +1,5 @@
 /* mod_auth_saml.c  -  Handwritten functions for Apache mod_auth_saml module
- * Copyright (c) 2012-2014 Synergetics NV (sampo@synergetics.be), All Rights Reserved.
+ * Copyright (c) 2012-2015 Synergetics NV (sampo@synergetics.be), All Rights Reserved.
  * Copyright (c) 2009-2011 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.
  * Copyright (c) 2008-2009 Symlabs (symlabs@symlabs.com), All Rights Reserved.
  * Author: Sampo Kellomaki (sampo@iki.fi)
@@ -19,6 +19,9 @@
  * 21.6.2013, added SOAP WSP capability --Sampo
  * 17.11.2013, move redir_to_content feature to zxid_simple() --Sampo
  * 8.2.2014,  added OPTIONAL_LOGIN_PAT feature --Sampo
+ * 5.3.2015,  improved Apache httpd-2.4 compatibility --Sampo
+ * 9.3.2015,  refactored to isolate httpd version dependencies to httpdglue.c --Sampo
+ * 20151218,  added special placeholder user "-anon-" for the 2.4 optional_login_pat case --Sampo
  *
  * To configure this module add to httpd.conf something like
  *
@@ -31,6 +34,12 @@
  *
  * http://httpd.apache.org/docs/2.2/developer/
  * http://modules.apache.org/doc/API.html
+ *
+ * Apache 2.4 Quirks
+ *
+ * [Mon Apr 13 23:04:07.291360 2015] [core:error] [pid 4841:tid 139900761208576] [client 127.0.0.1:60629] AH00027: No authentication done but request not allowed without authentication for /protected/index.html. Authentication not configured?
+ *
+ * See: httpd-2.4.12/server/request.c lines 250 and 287 (basically r->user needs to be set)
  */
 
 #define _LARGEFILE64_SOURCE   /* So off64_t is found, see: man 3 lseek64 */
@@ -52,15 +61,17 @@
 #include "ap_config.h"
 #include "ap_compat.h"
 #include "apr_strings.h"
-#include "httpd.h"
+#include "httpd.h"         /* request_rec et al. */
 #include "http_config.h"
 #include "http_core.h"
 #include "http_log.h"
 #include "http_protocol.h"
-#include "http_request.h"
+#include "http_request.h"  /* accessor methods for request_rec */
 
-#define srv_cf(s) (struct zxid_srv_cf*)ap_get_module_config((s)->module_config, &auth_saml_module)
-#define dir_cf(r) (zxid_conf*)ap_get_module_config((r)->per_dir_config, &auth_saml_module)
+#include "HRR.h"  /* httpd glue */
+
+/*#define srv_cf(s) (struct zxid_srv_cf*)ap_get_module_config((s)->module_config, &auth_saml_module)*/
+#define dir_cf(r) (zxid_conf*)ap_get_module_config(HRR_per_dir_config(r), &auth_saml_module)
 
 /* This extern variable is used as first argument to LoadModule in httpd.conf,
  * E.g: LoadModule auth_saml_module modules/mod_auth_saml.so */
@@ -77,7 +88,7 @@ static void chldinit(apr_pool_t* p, server_rec* s)
   D("server_rec=%p", m, s);
   res = curl_global_init(CURL_GLOBAL_ALL); /* vs. _SSL. Also OpenSSL vs. gnuTLS. */
   if(res != CURLE_OK) {
-    ap_log_error(APLOG_MARK, APLOG_ERR, 0, s, "Failed to initialize curl library: %u", res);
+    ERR("Failed to initialize curl library: %u", res);
   }
 }
 #endif
@@ -89,16 +100,16 @@ static void set_cookies(zxid_conf* cf, request_rec* r, const char* setcookie, co
   if (setcookie && setcookie[0] && setcookie[0] != '-') {
     /* http://dev.ariel-networks.com/apr/apr-tutorial/html/apr-tutorial-19.html */
     D("Set-Cookie(%s)", setcookie);
-    apr_table_addn(r->headers_out, "Set-Cookie", setcookie);
-    apr_table_addn(r->err_headers_out, "Set-Cookie", setcookie);  /* Only way to get redir to set header */
-    apr_table_addn(r->headers_in,  "Set-Cookie", setcookie);  /* So subrequest can pick them up! */
+    apr_table_addn(HRR_headers_out(r), "Set-Cookie", setcookie);
+    apr_table_addn(HRR_err_headers_out(r), "Set-Cookie", setcookie);  /* Only way to get redir to set header */
+    apr_table_addn(HRR_headers_in(r),  "Set-Cookie", setcookie);  /* So subrequest can pick them up! */
   }
   if (setptmcookie && setptmcookie[0] && setptmcookie[0] != '-') {
     /* http://dev.ariel-networks.com/apr/apr-tutorial/html/apr-tutorial-19.html */
     D("PTM Set-Cookie(%s)", setptmcookie);
-    apr_table_addn(r->headers_out, "Set-Cookie", setptmcookie);
-    apr_table_addn(r->err_headers_out, "Set-Cookie", setptmcookie);  /* Only way to get redir to set header */
-    apr_table_addn(r->headers_in,  "Set-Cookie", setptmcookie);  /* So subrequest can pick them up! */
+    apr_table_addn(HRR_headers_out(r), "Set-Cookie", setptmcookie);
+    apr_table_addn(HRR_err_headers_out(r), "Set-Cookie", setptmcookie);  /* Only way to get redir to set header */
+    apr_table_addn(HRR_headers_in(r),  "Set-Cookie", setptmcookie);  /* So subrequest can pick them up! */
   }
 }
 
@@ -130,7 +141,9 @@ static int pool2apache(zxid_conf* cf, request_rec* r, struct zxid_attr* pool)
   struct zxid_map* map;
   struct zxid_attr* at;
   struct zxid_attr* av;
-  
+  void* r_pool = HRR_pool(r);
+  void* sbe = HRR_subprocess_env(r);
+
   /* Length computation pass */
 
   for (at = pool; at; at = at->n) {
@@ -148,11 +161,11 @@ static int pool2apache(zxid_conf* cf, request_rec* r, struct zxid_attr* pool)
 	name = at->name;
       }
 
-      name = apr_psprintf(r->pool, "%s%s", cf->mod_saml_attr_prefix, name);
-      apr_table_set(r->subprocess_env, name, at->val);
+      name = apr_psprintf(r_pool, "%s%s", cf->mod_saml_attr_prefix, name);
+      apr_table_set(sbe, name, at->val);
       for (av = at->nv; av; av = av->n) {
 	av->map_val = zxid_map_val(cf, 0, 0, map, at->name, av->val);
-	apr_table_set(r->subprocess_env, name, av->map_val->s);
+	apr_table_set(sbe, name, av->map_val->s);
       }
     } else {
       if ((errmac_debug & ERRMAC_DEBUG_MASK)>1)
@@ -160,10 +173,10 @@ static int pool2apache(zxid_conf* cf, request_rec* r, struct zxid_attr* pool)
       else
 	D("ATTR(%s)=VAL(%.*s)", at->name, at->val?(int)MIN(35,strlen(at->val)):6, at->val?at->val:"(null)");
       /* *** handling of multivalued attributes (right now only last is preserved) */
-      name = apr_psprintf(r->pool, "%s%s", cf->mod_saml_attr_prefix, at->name);
-      apr_table_set(r->subprocess_env, name, at->val);
+      name = apr_psprintf(r_pool, "%s%s", cf->mod_saml_attr_prefix, at->name);
+      apr_table_set(sbe, name, at->val);
       for (av = at->nv; av; av = av->n)
-	apr_table_set(r->subprocess_env, name, av->val);
+	apr_table_set(sbe, name, av->val);
     }
     if      (!strcmp(at->name, "idpnid"))       idpnid = at->val;      /* Capture special */
     else if (!strcmp(at->name, "setcookie"))    setcookie = at->val;
@@ -172,6 +185,7 @@ static int pool2apache(zxid_conf* cf, request_rec* r, struct zxid_attr* pool)
     //else if (!strcmp(at->name, "rs"))         rs = at->val;
   }
 #if 0
+  /* This code moved to zxidsimp.c: zxid_show_protected_content_setcookie() */
   if (rs && rs[0] && rs[0] != '-') {
     /* N.B. RelayState was set by chkuid() "some other page" section by setting cgi.rs
      * to deflated and safe base64 encoded value which was then sent to IdP as RelayState.
@@ -184,12 +198,12 @@ static int pool2apache(zxid_conf* cf, request_rec* r, struct zxid_attr* pool)
     }
     rs_qs = strchr(rs, '?');
     if (rs_qs
-	?(memcmp(r->uri, rs, rs_qs-rs)||strcmp(r->args?r->args:"",rs_qs+1))
-	:strcmp(r->uri, rs)) {  /* Different, need external or internal redirect */
+	?(memcmp(HRR_uri(r), rs, rs_qs-rs)||strcmp(HRR_args(r)?HRR_args(r):"",rs_qs+1))
+	:strcmp(HRR_uri(r), rs)) {  /* Different, need external or internal redirect */
       D("redirect(%s) redir_to_content=%d", rs, cf->redir_to_content);
       //r->uri = apr_pstrdup(r->pool, val);
       if (cf->redir_to_content) {
-	apr_table_setn(r->headers_out, "Location", rs);
+	apr_table_setn(HRR_headers_out(r), "Location", rs);
 	ret = HTTP_SEE_OTHER;
       } else {
 	/* *** any attributes after this point may not appear in subrequest */
@@ -202,16 +216,17 @@ static int pool2apache(zxid_conf* cf, request_rec* r, struct zxid_attr* pool)
   set_cookies(cf, r, setcookie, setptmcookie);  
   if (cookie && cookie[0] != '-') {
     D("Cookie(%s) 2", cookie);
-    apr_table_addn(r->headers_in, "Cookie", cookie);  /* so internal redirect sees it */
+    apr_table_addn(HRR_headers_in(r), "Cookie", cookie);  /* so internal redirect sees it */
   }
   if (idpnid && idpnid[0] != '-') {
     D("REMOTE_USER(%s)", idpnid);
-    apr_table_set(r->subprocess_env, "REMOTE_USER", idpnid);
+    apr_table_set(sbe, "REMOTE_USER", idpnid);
+    HRR_set_user(r, idpnid);  /* httpd-2.4 anz framework requires this, 2.2 does not care */
   }
   
   //apr_table_setn(r->subprocess_env,
   //		 apr_psprintf(r->pool, "%sLDIF", cf->mod_saml_attr_prefix), ldif);
-  D("SSO OK ret(%d) uri(%s) filename(%s) path_info(%s)", ret, r->uri, r->filename, r->path_info);
+  D("SSO OK ret(%d) uri(%s) filename(%s) path_info(%s) user(%s)=%p", ret, (char*)HRR_uri(r), (char*)HRR_filename(r), (char*)HRR_path_info(r), STRNULLCHKD((char*)HRR_user(r)), HRR_user(r));
   return ret;
 }
 
@@ -226,16 +241,16 @@ static int send_res(zxid_conf* cf, request_rec* r, char* res)
   int len;
   char* p;
 #if 0
-  apr_table_setn(r->headers_out,     "Cache-Control", "no-cache");
-  apr_table_setn(r->err_headers_out, "Cache-Control", "no-cache");
-  apr_table_setn(r->headers_out,     "Pragma", "no-cache");
-  apr_table_setn(r->err_headers_out, "Pragma", "no-cache");
+  apr_table_setn(HRR_headers_out(r),     "Cache-Control", "no-cache");
+  apr_table_setn(HRR_err_headers_out(r), "Cache-Control", "no-cache");
+  apr_table_setn(HRR_headers_out(r),     "Pragma", "no-cache");
+  apr_table_setn(HRR_err_headers_out(r), "Pragma", "no-cache");
 #endif
   res += 14;  /* skip "Content-Type:" (14 chars) */
   DD("RES(%s)", res);
   p = strchr(res, '\r');
   *p = 0;
-  //apr_table_setn(r->headers_out, "Content-Type", res);
+  //apr_table_setn(HRR_headers_out(r), "Content-Type", res);
   DD("CONTENT-TYPE(%s)", res);
   ap_set_content_type(r, res);
   res = p+2 + 16;  /* skip "Content-Length:" (16 chars) */
@@ -248,8 +263,8 @@ static int send_res(zxid_conf* cf, request_rec* r, char* res)
   
   //register_timeout("send", r);
   ap_send_http_header(r);
-  if (!r->header_only)
-    ap_rputs(res, r);  //send_fd(f, r);  rprintf(); ap_rwrite()
+  if (!HRR_header_only(r))
+    ap_rprintf(r, "%s", res);  //send_fd(f, r);  rprintf(); ap_rwrite()
   return DONE;   /* Prevent further hooks from processing the request. */
 }
 
@@ -279,9 +294,9 @@ static char* read_post(zxid_conf* cf, request_rec* r)
   if (!ap_should_client_block(r)) {
     len = 0;
   } else {
-    len = r->remaining;
+    len = HRR_remaining(r);
   }
-  res = p = apr_palloc(r->pool, len + 1);
+  res = p = apr_palloc(HRR_pool(r), len + 1);
   res[len] = 0;
   D("remaining=%d", len);
   
@@ -290,7 +305,7 @@ static char* read_post(zxid_conf* cf, request_rec* r)
      * number of bytes otherwise.   */
     ret = ap_get_client_block(r, p, len);
     if (!ret) {
-      ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, "Failed to read POST data from client.");
+      ERR("Failed to read POST data from client. len=%d",len);
       return 0;  /* HTTP_INTERNAL_SERVER_ERROR */
     }
     
@@ -330,21 +345,26 @@ static int chkuid(request_rec* r)
   const char* cookie_hdr=0;
   const char* set_cookie_hdr;
   const char* cur_auth;
+  request_rec* main_req;
+  char* uri = r?HRR_uri(r):0;
   zxid_conf* cf = dir_cf(r);
   zxid_cgi cgi;
   zxid_ses ses;
   ZERO(&cgi, sizeof(zxid_cgi));
   ZERO(&ses, sizeof(zxid_ses));
-  cgi.uri_path = r?r->uri:0;
-  cgi.qs = r?r->args:0;
+  cgi.uri_path = uri;
+  cgi.qs = r?HRR_args(r):0;
 
-  D("===== START %s req=%p uri(%s) args(%s) pid=%d cwd(%s)", ZXID_REL, r, r?STRNULLCHKNULL(r->uri):"(r null)", r?STRNULLCHKNULL(r->args):"(r null)", getpid(), getcwd(buf,sizeof(buf)));
+  //D("request_rec sizeof=%d offset(r->uri)=%d offset(r->user)=%d", sizeof(request_rec), (void*)(uri)-(void*)r, (void*)(&(r->user))-(void*)r);
+  D("===== START %s req=%p uri(%s) args(%s) pid=%d cwd(%s)", ZXID_REL, r, r?STRNULLCHKNULL(uri):"(r null)", r?STRNULLCHKNULL(HRR_args(r)):"(r null)", getpid(), getcwd(buf,sizeof(buf)));
   if (cf->wd && *cf->wd)
     chdir(cf->wd);  /* Ensure the working dir is not / (sometimes Apache httpd changes dir) */
   D_INDENT("chkuid: ");
 
-  if (r->main) {  /* subreq can't come from net: always auth OK. */
-    D("sub ok %d", OK);
+  if (main_req = HRR_main(r)) {  /* subreq can't come from net: always auth OK. */
+    D("sub ok user(%s)=%p", STRNULLCHKD((char*)HRR_user(r)), HRR_user(r));
+    HRR_set_user(r, HRR_user(main_req));
+    D("sub from main user(%s)=%p", STRNULLCHKD((char*)HRR_user(r)), HRR_user(r));
     D_DEDENT("chkuid: ");
     return OK;
   }
@@ -355,56 +375,59 @@ static int chkuid(request_rec* r)
     D_DEDENT("chkuid: ");
     return DECLINED;
   }
-  r->ap_auth_type = "saml";
+  //r->ap_auth_type = "saml";  *** This is already verified to be the case?!?
   
   /* Probe for Session ID in cookie. Also propagate the cookie to subrequests. */
 
   if (cf->ses_cookie_name && *cf->ses_cookie_name) {
-    cookie_hdr = apr_table_get(r->headers_in, "Cookie");
+    cookie_hdr = apr_table_get(HRR_headers_in(r), "Cookie");
     if (cookie_hdr) {
       D("found cookie(%s) 3", STRNULLCHK(cookie_hdr));
       zxid_get_sid_from_cookie(cf, &cgi, cookie_hdr);
-      apr_table_addn(r->headers_out, "Cookie", cookie_hdr);       /* Pass cookies to subreq */
+      apr_table_addn(HRR_headers_out(r), "Cookie", cookie_hdr);       /* Pass cookies to subreq */
       DD("found cookie(%s) 5", STRNULLCHK(cookie_hdr));
       /* Kludge to get subrequest to set-cookie, i.e. on return path */
-      set_cookie_hdr = apr_table_get(r->headers_in, "Set-Cookie");
+      set_cookie_hdr = apr_table_get(HRR_headers_in(r), "Set-Cookie");
       if (set_cookie_hdr) {
 	D("subrequest set-cookie(%s) 2", set_cookie_hdr);
-	apr_table_addn(r->headers_out, "Set-Cookie", set_cookie_hdr);
+	apr_table_addn(HRR_headers_out(r), "Set-Cookie", set_cookie_hdr);
       }
     }
   }
 
   /* Redirect hack: deal with externally imposed ACS url that does not follow zxid convention. */
   
-  args_len = r->args?strlen(r->args):0;
-  if (cf->redirect_hack_imposed_url && !strcmp(r->uri, cf->redirect_hack_imposed_url)) {
-    D("Redirect hack: mapping(%s) imposed to zxid(%s)", r->uri, cf->redirect_hack_zxid_url);
-    r->uri = cf->redirect_hack_zxid_url;
+  args_len = HRR_args(r)?strlen(HRR_args(r)):0;
+  if (cf->redirect_hack_imposed_url && !strcmp(uri, cf->redirect_hack_imposed_url)) {
+    D("Redirect hack: mapping(%s) imposed to zxid(%s)", uri, cf->redirect_hack_zxid_url);
+    HRR_set_uri(r, cf->redirect_hack_zxid_url);
+    uri = cf->redirect_hack_zxid_url;
     if (cf->redirect_hack_zxid_qs && *cf->redirect_hack_zxid_qs) {
       if (args_len) {
 	/* concatenate redirect_hack_zxid_qs with existing qs */
 	len = strlen(cf->redirect_hack_zxid_qs);
-	cp = apr_palloc(r->pool, len+1+args_len+1);
+	cp = apr_palloc(HRR_pool(r), len+1+args_len+1);
 	strcpy(cp, cf->redirect_hack_zxid_qs);
 	cp[len] = '&';
-	strcpy(cp+len+1, r->args);
-	cgi.qs = r->args = cp;
+	strcpy(cp+len+1, HRR_args(r));
+	cgi.qs = cp;
+	HRR_set_args(r, cp);
       } else {
-	cgi.qs = r->args = cf->redirect_hack_zxid_qs;
+	cgi.qs = cf->redirect_hack_zxid_qs;
+	HRR_set_args(r, cgi.qs);
       }
-      args_len = strlen(r->args);
+      args_len = strlen(HRR_args(r));
     }
-    D("After hack uri(%s) args(%s)", STRNULLCHK(r->uri), STRNULLCHK(r->args));
+    D("After hack uri(%s) args(%s)", STRNULLCHK(uri), STRNULLCHK(HRR_args(r)));
   }
   
-  DD("HERE1 args_len=%d cgi=%p k(%s) args(%s)", args_len, &cgi, STRNULLCHKNULL(cgi.skin), STRNULLCHKNULL(r->args));
+  DD("HERE1 args_len=%d cgi=%p k(%s) args(%s)", args_len, &cgi, STRNULLCHKNULL(cgi.skin), STRNULLCHKNULL(HRR_args(r)));
   if (args_len) {
     /* leak the dup str: the cgi structure will take references to this and change &s to nuls */
-    cp = apr_palloc(r->pool, args_len + 1);
-    strcpy(cp, r->args);
+    cp = apr_palloc(HRR_pool(r), args_len + 1);
+    strcpy(cp, HRR_args(r));
     zxid_parse_cgi(cf, &cgi, cp);
-    DD("HERE2 args_len=%d cgi=%p k(%s) args(%s)", args_len, &cgi, STRNULLCHKNULL(cgi.skin), STRNULLCHKNULL(r->args));
+    DD("HERE2 args_len=%d cgi=%p k(%s) args(%s)", args_len, &cgi, STRNULLCHKNULL(cgi.skin), STRNULLCHKNULL(HRR_args(r)));
   }
   /* Check if we are supposed to enter zxid due to URL suffix - to
    * process protocol messages rather than ordinary pages. To do this
@@ -414,7 +437,7 @@ static int chkuid(request_rec* r)
    * else clause). Both then and else -clause URLs are defined as requiring
    * SSO by virtue of the web server configuration. */
 
-  uri_len = strlen(r->uri);
+  uri_len = strlen(uri);
   url_len = strlen(cf->burl);
   for (cp = cf->burl + url_len - 1; cp > cf->burl; --cp)
     if (*cp == '?')
@@ -422,9 +445,9 @@ static int chkuid(request_rec* r)
   if (cp == cf->burl)
     cp = cf->burl + url_len;
   
-  if (url_len >= uri_len && !memcmp(cp - uri_len, r->uri, uri_len)) {  /* Suffix match */
-    if (errmac_debug & MOD_AUTH_SAML_INOUT) INFO("matched uri(%s) cf->burl(%s) qs(%s) rs(%s) op(%c)", r->uri, cf->burl, STRNULLCHKNULL(r->args), STRNULLCHKNULL(cgi.rs), cgi.op);
-    if (r->method_number == M_POST) {
+  if (url_len >= uri_len && !memcmp(cp - uri_len, uri, uri_len)) {  /* Suffix match */
+    if (errmac_debug & MOD_AUTH_SAML_INOUT) INFO("matched uri(%s) cf->burl(%s) qs(%s) rs(%s) op(%c)", uri, cf->burl, STRNULLCHKNULL(HRR_args(r)), STRNULLCHKNULL(cgi.rs), cgi.op);
+    if (HRR_method_number(r) == M_POST) {
       res = read_post(cf, r);   /* Will print some debug output */
       if (res) {
 	if (cgi.op == 'S') {
@@ -464,73 +487,74 @@ static int chkuid(request_rec* r)
 	goto process_zxid_simple_outcome;
     }
     /* not logged in, fall thru */
-  } else if (zx_match(cf->wsp_pat, r->uri)) {
+  } else if (zx_match(cf->wsp_pat, uri)) {
     /* WSP case */
-    if (r->method_number == M_POST) {
+    if (HRR_method_number(r) == M_POST) {
       res = read_post(cf, r);   /* Will print some debug output */
       if (zxid_wsp_validate(cf, &ses, 0, res)) {
-	D("WSP(%s) request valid", r->uri);
-	D("WSP CALL uri(%s) filename(%s) path_info(%s)", r->uri, r->filename, r->path_info);
+	D("WSP(%s) request valid", uri);
+	D("WSP CALL uri(%s) filename(%s) path_info(%s)", uri, (char*)HRR_filename(r), (char*)HRR_path_info(r));
 	ret = pool2apache(cf, r, ses.at);
 	D_DEDENT("chkuid: ");
 	return ret;
 	/* Essentially we fall through and let CGI processing happen.
 	 * *** how to decorate the CGI return value?!? New hook needed? --Sampo */
       } else {
-	ERR("WSP(%s) request validation failed", r->uri);
+	ERR("WSP(%s) request validation failed", uri);
 	D_DEDENT("chkuid: ");
 	return HTTP_FORBIDDEN;
       }
     } else {
-      ERR("WSP(%s) must be called with POST method %d", r->uri, r->method_number);
+      ERR("WSP(%s) must be called with POST method %d", uri, HRR_method_number(r));
       D_DEDENT("chkuid: ");
       return HTTP_METHOD_NOT_ALLOWED;
     }
-  } else if (zx_match(cf->uma_pat, r->uri)) {
+  } else if (zx_match(cf->uma_pat, uri)) {
     /* UMA case */
-    if (r->method_number == M_POST) {
+    if (HRR_method_number(r) == M_POST) {
       res = read_post(cf, r);   /* Will print some debug output */
 #if 0
       // *** add UMA Resource Server stuff here
       if (zxid_wsp_validate(cf, &ses, 0, res)) {
-	D("WSP(%s) request valid", r->uri);
-	D("WSP CALL uri(%s) filename(%s) path_info(%s)", r->uri, r->filename, r->path_info);
+	D("WSP(%s) request valid", uri);
+	D("WSP CALL uri(%s) filename(%s) path_info(%s)", uri, HRR_filename(r), HRR_path_info(r));
 	ret = pool2apache(cf, r, ses.at);
 	D_DEDENT("chkuid: ");
 	return ret;
 	/* Essentially we fall through and let CGI processing happen.
 	 * *** how to decorate the CGI return value?!? New hook needed? --Sampo */
       } else {
-	ERR("WSP(%s) request validation failed", r->uri);
+	ERR("WSP(%s) request validation failed", uri);
 	D_DEDENT("chkuid: ");
 	return HTTP_FORBIDDEN;
       }
 #endif
     } else {
-      ERR("WSP(%s) must be called with POST method %d", r->uri, r->method_number);
+      ERR("WSP(%s) must be called with POST method %d", uri, HRR_method_number(r));
       D_DEDENT("chkuid: ");
       return HTTP_METHOD_NOT_ALLOWED;
     }
   } else {
     /* Some other page. Just check for session. */
-    if (errmac_debug & MOD_AUTH_SAML_INOUT) INFO("other page uri(%s) qs(%s) cf->burl(%s) uri_len=%d url_len=%d", r->uri, STRNULLCHKNULL(r->args), cf->burl, uri_len, url_len);
+    if (errmac_debug & MOD_AUTH_SAML_INOUT) INFO("other page uri(%s) qs(%s) cf->burl(%s) uri_len=%d url_len=%d", uri, STRNULLCHKNULL(HRR_args(r)), cf->burl, uri_len, url_len);
     if (cgi.sid && cgi.sid[0] && zxid_get_ses(cf, &ses, cgi.sid)) {
       res = zxid_simple_ses_active_cf(cf, &cgi, &ses, 0, AUTO_FLAGS);
       if (res)
 	goto process_zxid_simple_outcome;
     } else {
       D("No active session(%s) op(%c)", STRNULLCHK(cgi.sid), cgi.op?cgi.op:'-');
-      if (cf->optional_login_pat && zx_match(cf->optional_login_pat, r->uri)) {
-	D("optional_login_pat matches ok %d", OK);
+      if (cf->optional_login_pat && zx_match(cf->optional_login_pat, uri)) {
+	D("optional_login_pat matches %d", OK);
+	HRR_set_user(r, "-anon-");  /* httpd-2.4 anz framework requires this, 2.2 does not care */
 	D_DEDENT("chkuid: ");
 	return OK;
       }
     }
-    if (r->args && r->args[0] == 'l') {
-      D("Detect login(%s)", r->args);
+    if (HRR_args(r) && ((char*)HRR_args(r))[0] == 'l') {
+      D("Detect login(%s)", (char*)HRR_args(r));
     } else
       cgi.op = 'E';   /* Trigger IdP selection screen */
-    D("other page: no_ses uri(%s) templ(%s) tf(%s) k(%s)", r->uri, STRNULLCHKNULL(cgi.templ), STRNULLCHKNULL(cf->idp_sel_templ_file), STRNULLCHKNULL(cgi.skin));
+    D("other page: no_ses uri(%s) templ(%s) tf(%s) k(%s)", uri, STRNULLCHKNULL(cgi.templ), STRNULLCHKNULL(cf->idp_sel_templ_file), STRNULLCHKNULL(cgi.skin));
   }
 step_up:
   res = zxid_simple_no_ses_cf(cf, &cgi, &ses, 0, AUTO_FLAGS);
@@ -544,7 +568,7 @@ process_zxid_simple_outcome:
   switch (res[0]) {
   case 'L':
     if (errmac_debug & MOD_AUTH_SAML_INOUT) INFO("REDIR(%s)", res);
-    apr_table_setn(r->headers_out, "Location", res+10);
+    apr_table_setn(HRR_headers_out(r), "Location", res+10);
     set_cookies(cf, r, ses.setcookie, ses.setptmcookie);  
     D_DEDENT("chkuid: ");
     return HTTP_SEE_OTHER;
@@ -559,14 +583,14 @@ process_zxid_simple_outcome:
     D_DEDENT("chkuid: ");
     return HTTP_FORBIDDEN;
   case 0: /* Logged in case */
-    D("SSO OK pre uri(%s) filename(%s) path_info(%s)", r->uri, r->filename, r->path_info);
+    D("SSO OK pre uri(%s) filename(%s) path_info(%s)", uri, (char*)HRR_filename(r), (char*)HRR_path_info(r));
     ret = pool2apache(cf, r, ses.at);
     D_DEDENT("chkuid: ");
     return ret;
 #if 0
   case 'd': /* Logged in case */
     if (errmac_debug & MOD_AUTH_SAML_INOUT) INFO("SSO OK LDIF(%s)", res);
-    D("SSO OK pre uri(%s) filename(%s) path_info(%s)", r->uri, r->filename, r->path_info);
+    D("SSO OK pre uri(%s) filename(%s) path_info(%s)", uri, (char*)HRR_filename(r), (char*)HRR_path_info(r));
     ret = ldif2apache(cf, r, res);
     D_DEDENT("chkuid: ");
     return ret;
@@ -634,7 +658,9 @@ const command_rec zxid_apache_commands[] = {
 /*(-) Create default configuration in response for Apache <Location> or <Directory>
  * directives. This is then augmented by ZXIDConf directives.
  * This code may run twice: once for syntax check, and then again for
- * production use. Currently we just redo the work.
+ * production use. Currently we just redo the work. Apache stores the
+ * return value of this function and it can be read in chkuid() hook using
+ *    ap_get_module_config((r)->per_dir_config, &auth_saml_module)
  *
  * This is considered internal function to mod_auth_saml. Do not call directly. */
 
@@ -661,7 +687,7 @@ static void* dirconf(apr_pool_t* p, char* d)
  * This is considered internal function to mod_auth_saml. Do not call directly. */
 
 /* Called by: */
-static void reghk(apr_pool_t *p) {
+static void reghk(apr_pool_t* p) {
   D("pool=%p", p);
   //ap_hook_access_checker(authusr,  0, 0, APR_HOOK_MIDDLE);
   ap_hook_check_user_id( chkuid,   0, 0, APR_HOOK_MIDDLE);
@@ -670,7 +696,10 @@ static void reghk(apr_pool_t *p) {
 }
 
 /* This extern variable is used as first argument to LoadModule in httpd.conf,
- * E.g: LoadModule auth_saml_module modules/mod_auth_saml.so */
+ * E.g: LoadModule auth_saml_module modules/mod_auth_saml.so
+ * See httpd-2.2/include/http_config.h for module_struct.
+ * Lucky for 2.2 vs. 2.4 compat, m->module_index and other fields up to
+ * m->magic are on sample places on both. */
 
 module AP_MODULE_DECLARE_DATA auth_saml_module = {
   STANDARD20_MODULE_STUFF,
