@@ -15,7 +15,8 @@
  * 23.2.2008, added support for function pointers to malloc(),
  *            realloc(), and free() --Sampo
  * 27.10.2010, namespace re-engineering --Sampo
- * 26.10.2014, changed crypto to GFC and OAEP to combat Backwards Compatibility Attacks --Sampo
+ * 26.10.2014, changed crypto to GCM and OAEP to combat Backwards Compatibility Attacks --Sampo
+ * 18.12.2015, applied patch from soconnor, perceptyx, adding algos --Sampo
  *
  * See paper: Tibor Jager, Kenneth G. Paterson, Juraj Somorovsky: "One Bad Apple: Backwards Compatibility Attacks on State-of-the-Art Cryptography", 2013 http://www.nds.ruhr-uni-bochum.de/research/publications/backwards-compatibility/ /t/BackwardsCompatibilityAttacks.pdf
  *
@@ -237,7 +238,7 @@ void zx_free_ctx(struct zx_ctx* ctx);	/* Wrapper for free(3C). */
 #endif
 
 /* Special token values. */
-#define ZX_TOK_NO_ATTR   (-7)
+#define ZX_TOK_NO_ATTR   (-7)  /* 0xfff9 65529 */
 #define ZX_TOK_ATTR_ERR  (-6)
 #define ZX_TOK_XMLNS     (-4)
 #define ZX_TOK_DATA             0x0000fffd  /* Decimal 65533: string data between elements */
@@ -299,7 +300,8 @@ int zx_format_parse_error(struct zx_ctx* ctx, char* buf, int siz, char* logkey);
 
 #define ZX_SYMKEY_LEN 20  /* size of sha1 */
 char* zx_hmac_sha256(struct zx_ctx* c, int key_len, const char* key, int data_len, const char* data, char* md, int* md_len);
-char* zx_raw_digest2(struct zx_ctx* c, char* md, char* const algo, int len, const char* s, int len2, const char* s2);
+int zx_raw_raw_digest2(struct zx_ctx* c, char* mdbuf, const EVP_MD* evp_digest, int len, const char* s, int len2, const char* s2);
+int zx_raw_digest2(struct zx_ctx* c, char* mdbuf, const char* algo, int len, const char* s, int len2, const char* s2);
 struct zx_str* zx_raw_cipher(struct zx_ctx* c, const char* algo, int encflag, struct zx_str* key, int len, const char* s, int iv_len, const char* iv);
 struct zx_str* zx_rsa_pub_enc(struct zx_ctx* c, struct zx_str* plain, RSA* rsa_pkey, int pad);
 struct zx_str* zx_rsa_pub_dec(struct zx_ctx* c, struct zx_str* ciphered, RSA* rsa_pkey, int pad);
@@ -356,19 +358,57 @@ struct zx_root_s* zx_dec_zx_root(struct zx_ctx* c, int len, const char* start, c
 void zx_DEC_elem(struct zx_ctx* c, struct zx_elem_s* x);
 struct zx_el_desc* zx_el_desc_lookup(int tok);
 
-#define SIG_ALGO_RSA_SHA1  "http://www.w3.org/2000/09/xmldsig#rsa-sha1"
-#define SIG_ALGO_DSA_SHA1  "http://www.w3.org/2000/09/xmldsig#dsa-sha1"
-#define SIG_ALGO    SIG_ALGO_RSA_SHA1
-#define DIGEST_ALGO        "http://www.w3.org/2000/09/xmldsig#sha1"
+#define SIG_ALGO_RSA_SHA1_URLENC   "http://www.w3.org/2000/09/xmldsig%23rsa-sha1"
+#define SIG_ALGO_RSA_SHA224_URLENC "http://www.w3.org/2001/04/xmldsig-more%23rsa-sha224"
+#define SIG_ALGO_RSA_SHA256_URLENC "http://www.w3.org/2001/04/xmldsig-more%23rsa-sha256"
+#define SIG_ALGO_RSA_SHA384_URLENC "http://www.w3.org/2001/04/xmldsig-more%23rsa-sha384"
+#define SIG_ALGO_RSA_SHA512_URLENC "http://www.w3.org/2001/04/xmldsig-more%23rsa-sha512"
+#define SIG_ALGO_DSA_SHA1_URLENC   "http://www.w3.org/2000/09/xmldsig%23dsa-sha1"
+#define SIG_ALGO_DSA_SHA224_URLENC "http://www.w3.org/2001/04/xmldsig-more%23dsa-sha224"
+#define SIG_ALGO_DSA_SHA256_URLENC "http://www.w3.org/2001/04/xmldsig-more%23dsa-sha256"
+#define SIG_ALGO_DSA_SHA384_URLENC "http://www.w3.org/2001/04/xmldsig-more%23dsa-sha384"
+#define SIG_ALGO_DSA_SHA512_URLENC "http://www.w3.org/2001/04/xmldsig-more%23dsa-sha512"
+#define SIG_ALGO_ECDSA_SHA1_URLENC   "http://www.w3.org/2001/04/xmldsig-more%23ecdsa-sha1"
+#define SIG_ALGO_ECDSA_SHA224_URLENC "http://www.w3.org/2001/04/xmldsig-more%23ecdsa-sha224"
+#define SIG_ALGO_ECDSA_SHA256_URLENC "http://www.w3.org/2001/04/xmldsig-more%23ecdsa-sha256"
+#define SIG_ALGO_ECDSA_SHA384_URLENC "http://www.w3.org/2001/04/xmldsig-more%23ecdsa-sha384"
+#define SIG_ALGO_ECDSA_SHA512_URLENC "http://www.w3.org/2001/04/xmldsig-more%23ecdsa-sha512"
+
+#define SIG_ALGO_RSA_SHA1   "http://www.w3.org/2000/09/xmldsig#rsa-sha1"
+#define SIG_ALGO_RSA_SHA224 "http://www.w3.org/2001/04/xmldsig-more#rsa-sha224"
+#define SIG_ALGO_RSA_SHA256 "http://www.w3.org/2001/04/xmldsig-more#rsa-sha256"
+#define SIG_ALGO_RSA_SHA384 "http://www.w3.org/2001/04/xmldsig-more#rsa-sha384"
+#define SIG_ALGO_RSA_SHA512 "http://www.w3.org/2001/04/xmldsig-more#rsa-sha512"
+#define SIG_ALGO_DSA_SHA1   "http://www.w3.org/2000/09/xmldsig#dsa-sha1"
+#define SIG_ALGO_DSA_SHA224 "http://www.w3.org/2009/xmldsig11#dsa-sha224"
+#define SIG_ALGO_DSA_SHA256 "http://www.w3.org/2009/xmldsig11#dsa-sha256"
+#define SIG_ALGO_DSA_SHA384 "http://www.w3.org/2009/xmldsig11#dsa-sha384"
+#define SIG_ALGO_DSA_SHA512 "http://www.w3.org/2009/xmldsig11#dsa-sha512"
+#define SIG_ALGO_ECDSA_SHA1   "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha1"
+#define SIG_ALGO_ECDSA_SHA224 "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha224"
+#define SIG_ALGO_ECDSA_SHA256 "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha256"
+#define SIG_ALGO_ECDSA_SHA384 "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha384"
+#define SIG_ALGO_ECDSA_SHA512 "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha512"
+
+#define DIGEST_ALGO_SHA1   "http://www.w3.org/2000/09/xmldsig#sha1"
+#define DIGEST_ALGO_SHA224 "http://www.w3.org/2001/04/xmldsig-more#sha224"
+#define DIGEST_ALGO_SHA256 "http://www.w3.org/2001/04/xmlenc#sha256"
+#define DIGEST_ALGO_SHA384 "http://www.w3.org/2001/04/xmldsig-more#sha384"
+#define DIGEST_ALGO_SHA512 "http://www.w3.org/2001/04/xmlenc#sha512"
+
+#define SIG_ALGO        SIG_ALGO_RSA_SHA1
+#define SIG_ALGO_URLENC SIG_ALGO_RSA_SHA1_URLENC
+#define SIG_SIZE 1024  /* Maximum size of the base64 encoded signature, for buffer allocation */
+#define DIGEST_ALGO     DIGEST_ALGO_SHA1
 #define CANON_ALGO         "http://www.w3.org/2001/10/xml-exc-c14n#"
 #define ENVELOPED_ALGO     "http://www.w3.org/2000/09/xmldsig#enveloped-signature"
 #define ENC_ALGO_TRIPLEDES_CBC "http://www.w3.org/2001/04/xmlenc#tripledes-cbc"
 #define ENC_ALGO_AES128_CBC    "http://www.w3.org/2001/04/xmlenc#aes128-cbc"
 #define ENC_ALGO_AES192_CBC    "http://www.w3.org/2001/04/xmlenc#aes192-cbc"
 #define ENC_ALGO_AES256_CBC    "http://www.w3.org/2001/04/xmlenc#aes256-cbc"
-#define ENC_ALGO_AES256_GFC    "http://www.w3.org/2001/04/xmlenc#aes256-gfc"
+#define ENC_ALGO_AES256_GCM    "http://www.w3.org/2001/04/xmlenc11#aes256-gcm"
 /* #define ENC_ALGO            ENC_ALGO_AES128_CBC  unsafe, see Backwards Compatibility Attacks */
-#define ENC_ALGO               ENC_ALGO_AES256_GFC
+#define ENC_ALGO               ENC_ALGO_AES256_GCM
 
 /* The ENC_KEYTRAN_ALGO setting must agree with setting in zxenc_pubkey_enc()
  * See paper: Tibor Jager, Kenneth G. Paterson, Juraj Somorovsky: "One Bad Apple: Backwards Compatibility Attacks on State-of-the-Art Cryptography", 2013 http://www.nds.ruhr-uni-bochum.de/research/publications/backwards-compatibility/ /t/BackwardsCompatibilityAttacks.pdf
