@@ -48,7 +48,7 @@
 extern int verbose;  /* defined in option parsing in zxbusd.c */
 extern char* zxbus_path;
 
-/*() Allocate an entity
+/*() Allocate a bus entity
  * locking:: must be called inside shf->ent_mut
  * return:: pointer to hi_ent on success, 0 on failure */
 
@@ -68,7 +68,14 @@ struct hi_ent* zxbus_new_ent(struct hiios* shf, int len, const char* eid)
   return ent;
 }
 
-/*() Load an entity (and check that it exists)
+/*() Allocate a bus entity (and check that it exists)
+ * The bus entities are special users in /var/zxid/bus/uid/
+ * hierarchy. The user name is the succinct id formed
+ * by sha1-base64 of the Entity ID. New bus entities can be provisioned
+ * using the zxpasswd(1) tool, e.g.
+ *
+ *   echo -n 'pw123' | ./zxpasswd -new 2E_uLovDu748vn9dWEM6tqVzqUQ /var/zxid/bus/uid/
+ *
  * locking:: must be called inside shf->ent_mut
  * return:: pointer to hi_ent on success, 0 on failure */
 
@@ -147,7 +154,7 @@ static int zxbus_pw_authn_ent(const char* eid, const char* passw, int fd_hint)
   memcpy(pw_buf, passw, len);
   pw_buf[len] = 0;
   
-  return zx_password_authn(zxbus_path, sha1_name, pw_buf, fd_hint);
+  return zx_password_authn(zxbus_path, sha1_name, pw_buf, 0, fd_hint);
   /* *** add password overwrite in memory */
 }
 
@@ -160,6 +167,7 @@ static int zxbus_pw_authn_ent(const char* eid, const char* passw, int fd_hint)
  *
  * To create bus users, which use SHA1 of their EntityID (the entityID is passed
  * in STOMP 1.1 login header) as username, you should follow these steps
+ *
  * 1. Run ./zxbuslist -c 'URL=https://sp.foo.com/' -dc to determine the entity ID
  * 2. Convert entity ID to SHA1 hash: ./zxcot -p 'http://sp.foo.com?o=B'
  * 3. Create the user: ./zxpasswd -at 'eid: http://sp.foo.com?o=B' -new G2JpTSX_dbdJ7frhYNpKWGiMdTs /var/zxid/bus/uid/ <passwd
