@@ -1,4 +1,5 @@
 /* zxid/zxcrypto.c  -  Glue for cryptographical functions
+ * Copyright (c) 2015-2016 Synergetics NV (sampo@synergetics.be), All Rights Reserved.
  * Copyright (c) 2011 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.
  * Copyright (c) 2006-2009 Symlabs (symlabs@symlabs.com), All Rights Reserved.
  * Author: Sampo Kellomaki (sampo@iki.fi)
@@ -11,8 +12,13 @@
  * 7.10.2008, added documentation --Sampo
  * 29.8.2009, added zxid_mk_self_signed_cert() --Sampo
  * 12.12.2011, added HMAC SHA-256 as needed by JWT/JWS --Sampo
+ * 6.6.2015,   added aes-256-gcm --Sampo
+ * 16.10.2015, upgraded sha256 support, eliminated MD5 from certs --Sampo
  *
  * See paper: Tibor Jager, Kenneth G. Paterson, Juraj Somorovsky: "One Bad Apple: Backwards Compatibility Attacks on State-of-the-Art Cryptography", 2013 http://www.nds.ruhr-uni-bochum.de/research/publications/backwards-compatibility/ /t/BackwardsCompatibilityAttacks.pdf
+ *
+ * http://wiki.openssl.org/index.php/EVP_Authenticated_Encryption_and_Decryption
+ * https://www.openssl.org/docs/crypto/EVP_EncryptInit.html#gcm_and_ocb_modes
  */
 
 #include "platform.h"  /* needed on Win32 for snprintf() et al. */
@@ -28,6 +34,7 @@
 #ifdef USE_OPENSSL
 #include <openssl/evp.h>
 #include <openssl/md5.h>
+#include <openssl/sha.h>
 #include <openssl/hmac.h>
 #include <openssl/rand.h>
 #include <openssl/x509.h>
@@ -62,56 +69,73 @@ int zx_EVP_CIPHER_iv_length(const EVP_CIPHER* cipher)  { return EVP_CIPHER_iv_le
 int zx_EVP_CIPHER_block_size(const EVP_CIPHER* cipher) { return EVP_CIPHER_block_size(cipher); }
 #endif
 
+/*() Get certificate signature algorithm string. This reads the
+ * signature algorithm from certificate itself.
+ * Returns something like "SHA1" or "SHA256" or "" on error.
+ */
+
+/* Called by:  */
+const char* zxid_get_cert_signature_algo(X509* cert)
+{
+    if (!cert)
+       return "";
+    return OBJ_nid2ln(OBJ_obj2nid(cert->sig_alg->algorithm));
+}
+
 /*() zx_raw_digest2() computes a message digest over two items. The result
  * is placed in buffer md, which must already be of length sufficient for
  * the digest. md will not be nul terminated (and will usually have binary
- * data). Possible algos: SHA1 */
+ * data). Possible algos: "SHA1", "SHA256", "SHA512", etc.
+ * zx_raw_raw_digest() expects an algorithm object instead of a string.
+ * Returns 0 on failure or length of the digest on success.  */
 
-char* zx_raw_digest2(struct zx_ctx* c, char* md, char* const algo, int len, const char* s, int len2, const char* s2)
+int zx_raw_raw_digest2(struct zx_ctx* c, char* md, const EVP_MD* evp_digest, int len, const char* s, int len2, const char* s2)
 {
-  char* where = "start";
-  const EVP_MD* evp_digest;
-  EVP_MD_CTX ctx;
-  OpenSSL_add_all_digests();
-  EVP_MD_CTX_init(&ctx);
-  evp_digest = EVP_get_digestbyname(algo);
-  if (!evp_digest) {
-    ERR("Digest algo name(%s) not recognized by the crypto library (OpenSSL)", algo);
-    return 0;
-  }
+  char* where = "a";
+  EVP_MD_CTX* mdctx;
+  unsigned int mdlen;
+  mdctx = EVP_MD_CTX_create();
     
-  if (!EVP_DigestInit_ex(&ctx, evp_digest, 0 /* engine */)) {
+  if (!EVP_DigestInit_ex(mdctx, evp_digest, 0 /* engine */)) {
     where = "EVP_DigestInit_ex()";
     goto sslerr;
   }
   
   if (len && s) {
-    if (!EVP_DigestUpdate(&ctx, s, len)) {
+    if (!EVP_DigestUpdate(mdctx, s, len)) {
       where = "EVP_DigestUpdate()";
       goto sslerr;
     }
   }
   
   if (len2 && s2) {
-    if (!EVP_DigestUpdate(&ctx, s2, len2)) {
+    if (!EVP_DigestUpdate(mdctx, s2, len2)) {
       where = "EVP_DigestUpdate() 2";
       goto sslerr;
     }
   }
   
-  if(!EVP_DigestFinal_ex(&ctx, (unsigned char*)md, 0)) {
+  if(!EVP_DigestFinal_ex(mdctx, (unsigned char*)md, &mdlen)) {
     where = "EVP_DigestFinal_ex()";
     goto sslerr;
   }
-  EVP_MD_CTX_cleanup(&ctx);
-  return md;
+  EVP_MD_CTX_destroy(mdctx);
+  return mdlen;
 
  sslerr:
   zx_report_openssl_err(where);
-  EVP_MD_CTX_cleanup(&ctx);
+  EVP_MD_CTX_destroy(mdctx);
   return 0;
 }
 
+int zx_raw_digest2(struct zx_ctx* c, char* md, const char* algo, int len, const char* s, int len2, const char* s2)
+{
+  const EVP_MD* evp_digest;
+  OpenSSL_add_all_digests();
+  evp_digest = EVP_get_digestbyname(algo);
+  return zx_raw_raw_digest2(c, md, evp_digest, len, s, len2, s2);
+}
+
 /*() zx_EVP_DecryptFinal_ex() is a drop-in replacement for OpenSSL EVP_DecryptFinal_ex.
  * It performs XML Enc compatible padding check.  See OpenSSL bug 1067
  * http://rt.openssl.org/Ticket/Display.html?user=guest&;pass=guest&id=1067 */
@@ -168,10 +192,12 @@ struct zx_str* zx_raw_cipher(struct zx_ctx* c, const char* algo, int encflag, st
   const char* ivv;
   char* where = "start";
   struct zx_str* out;
-  int outlen, tmplen, alloclen;
+  int outlen=0, tmplen, alloclen;
   const EVP_CIPHER* evp_cipher;
   EVP_CIPHER_CTX ctx;
   OpenSSL_add_all_algorithms();
+  if ((errmac_debug&ERRMAC_DEBUG_MASK) > 2) hexdmp("plain  ", s, len, 256);  
+  D("len=%d s=%p", len, s);
   EVP_CIPHER_CTX_init(&ctx);
   evp_cipher = EVP_get_cipherbyname(algo);
   if (!evp_cipher) {
@@ -188,37 +214,49 @@ struct zx_str* zx_raw_cipher(struct zx_ctx* c, const char* algo, int encflag, st
       }
       ivv = iv;
     } else {
+      iv_len = tmplen;
       ivv = ZX_DEFAULT_IV;
       ASSERTOPI(EVP_MAX_IV_LENGTH, <=, sizeof(ZX_DEFAULT_IV));
     }
+    if ((errmac_debug&ERRMAC_DEBUG_MASK) > 2) hexdmp("iv     ", ivv, iv_len, 1024);
   } else
     ivv = 0;
+  if ((errmac_debug&ERRMAC_DEBUG_MASK) > 2) hexdmp("symkey ", key->s, key->len, 1024);
   
+#if 0
   alloclen = EVP_CIPHER_block_size(evp_cipher);
   alloclen = len + alloclen + alloclen;  /* bit pessimistic, but man EVP_CipherInit is ambiguous about the actual size needed. */
+#else
+  /* 20150606, it appears aes-256-gcm reports too short block size, thus we impose a minimum. */
+  alloclen = EVP_CIPHER_block_size(evp_cipher);
+  D("block_size=%d", alloclen);
+  alloclen = MAX(alloclen, 256);
+  alloclen = len + alloclen + alloclen;  /* bit pessimistic, but man EVP_CipherInit is ambiguous about the actual size needed. */
+#endif
   if (encflag)
     alloclen += iv_len;
   
   out = zx_new_len_str(c, alloclen);
+  D("alloclen=%d iv_len=%d encflag=%d out=%p iv=%p", alloclen, iv_len, encflag, out, iv);
   if (!out) goto clean;
   if (encflag)
     memcpy(out->s, ivv, iv_len);
   else
     iv_len = 0;  /* When decrypting, the iv has already been stripped. */
-  
-  if ((errmac_debug&ERRMAC_DEBUG_MASK) > 2) hexdmp("symkey ", key->s, key->len, 1024);
-  
+    
   if (!EVP_CipherInit_ex(&ctx, evp_cipher, 0 /* engine */, (unsigned char*)key->s, (unsigned char*)ivv, encflag)) {
     where = "EVP_CipherInit_ex()";
     goto sslerr;
   }
   
   if (!EVP_CIPHER_CTX_set_key_length(&ctx, key->len)) {
+    D("key->len=%d", key->len);
     where = "wrong key length for algorithm (block ciphers only accept keys of determined length)";
     goto sslerr;
   }
   
   if (!EVP_CipherUpdate(&ctx, (unsigned char*)out->s + iv_len, &outlen, (unsigned char*)s, len)) { /* Actual crypto happens here */
+    D("len=%d s=%p iv_len=%d outlen=%d out->s=%p", len, s, iv_len, outlen, out->s);
     where = "EVP_CipherUpdate()";
     goto sslerr;
   }
@@ -254,9 +292,11 @@ struct zx_str* zx_raw_cipher(struct zx_ctx* c, const char* algo, int encflag, st
   ASSERTOPI(outlen + iv_len, <=, alloclen);
   out->len = outlen + iv_len;
   out->s[outlen + iv_len] = 0;  /* nul term */
+  if ((errmac_debug&ERRMAC_DEBUG_MASK) > 2) hexdmp("cipher ", out->s, out->len, 256);  
   return out;
 
  sslerr:
+  D("where(%s)", where);
   zx_report_openssl_err(where);
  clean:
   EVP_CIPHER_CTX_cleanup(&ctx);
@@ -530,6 +570,7 @@ int zxid_mk_self_sig_cert(zxid_conf* cf, int buflen, char* buf, const char* lk,
 #if 0
   rsa = RSA_generate_key(1024 /*bits*/, 0x10001 /*65537*/, 0 /*req_cb*/, 0 /*arg*/);
 #else
+  /* Crypto analysis (2015) suggests 1024bit key is too weak. */
   rsa = RSA_generate_key(2048 /*bits*/, 0x10001 /*65537*/, 0 /*req_cb*/, 0 /*arg*/);
 #endif
   DD("keygen rsa key generated %s", name);
@@ -615,8 +656,14 @@ badurl:
   /*req->req_info->req_kludge=0;    / * no asn1 kludge *** filed deleted as of 0.9.7b?!? */
   
   DD("keygen signing request %s", lk);
+#if 0
   X509_REQ_sign(req, pkey, EVP_md5());
-  
+#else
+  /* Due to recent (2013) progress in crypto analysis, MD5 and SHA1 are considered
+   * weak and support is likely to be discontinued in browsers and operating systems. */
+  X509_REQ_sign(req, pkey, EVP_sha256());
+#endif
+
   /* ----- X509 create self signed certificate ----- */
   
   DD("keygen making x509ss %s", lk);
@@ -658,7 +705,12 @@ badurl:
   X509_add_ext(x509ss, ext, -1);
   
   DD("keygen signing x509ss %s", lk);
-  if (!(X509_sign(x509ss, pkey, EVP_md5()))) {
+#if 0
+  if (!(X509_sign(x509ss, pkey, EVP_md5())))
+#else
+  if (!(X509_sign(x509ss, pkey, EVP_sha256())))
+#endif
+  {
     ERR("Failed to sign x509ss %s", lk);
     zx_report_openssl_err("X509_sign");
     return 0;
@@ -916,7 +968,12 @@ int zxid_mk_at_cert(zxid_conf* cf, int buflen, char* buf, const char* lk, zxid_n
   zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "mk_at_cert");
 
   DD("keygen signing x509ss %s", lk);
-  if (!(X509_sign(x509ss, sign_pkey, EVP_md5()))) {
+#if 0
+  if (!(X509_sign(x509ss, sign_pkey, EVP_md5())))
+#else
+  if (!(X509_sign(x509ss, sign_pkey, EVP_sha256())))
+#endif
+  {
     ERR("Failed to sign x509ss %s", lk);
     zx_report_openssl_err("X509_sign");
     return 0;
