@@ -24,6 +24,7 @@
  * 16.2.2013,  added WD option --Sampo
  * 14.3.2013   added language/skin dependent templates --Sampo
  * 21.6.2013,  added wsp_pat --Sampo
+ * 18.12.2015, applied patch from soconnor, perceptyx --Sampo
  */
 
 #ifndef _zxid_h
@@ -42,8 +43,8 @@
 /*(c) ZXID configuration and working directory path
  * Where metadata cache and session files are created. Note that the directory
  * is not hashed: you should use a file system that scales easily to oodles
- * of small files in one directory. Say `make dir' to create the directory
- * with proper layout. If you change it here, also edit Makefile. */
+ * of small files in one directory. Say `zxcot -dirs' (or `make dir') to create
+ * the directory with proper layout. If you change it here, also edit Makefile. */
 #ifndef ZXID_PATH
 #ifdef MINGW
 #define ZXID_PATH  "c:/var/zxid/"
@@ -253,7 +254,7 @@ struct zxid_conf {
   char* anon_ok;
   char* optional_login_pat;
   char** required_authnctx;  /* Array of acceptable authentication context class refs */
-  char* issue_authnctx_pw;   /* What authentication context IdP issues for password authent. */
+  struct zxid_cstr_list* issue_authnctx;  /* What authentication context IdP issues for for different authentication methods. */
   char* idp_pref_acs_binding;
   char* mandatory_attr;
   int   before_slop;
@@ -296,6 +297,8 @@ struct zxid_conf {
   struct zxid_obl_list* wsp_localpdp_obl_req;
   char* wsp_localpdp_obl_emit;
   struct zxid_obl_list* wsc_localpdp_obl_accept;
+
+  struct zxid_map*   unix_grp_az_map;
   
   int   bootstrap_level;     /* How many layers of bootstraps are generated. */
   int   max_soap_retry;      /* How many times a ID-WSF SOAP call can be retried (update EPR) */
@@ -342,7 +345,6 @@ struct zxid_conf {
   char  log_act;
   char  log_issue_a7n;
   char  log_issue_msg;
-
   char  log_rely_a7n;
   char  log_rely_msg;
   char  log_err_in_act;      /* Log errors to action log flag (may also log to error log) */
@@ -352,7 +354,6 @@ struct zxid_conf {
   char  log_level;           /* act log level: 0=audit, 1=audit+extio, 2=audit+extio+events */
   char  user_local;          /* Whether local user accounts should be maintained. */
   char  redir_to_content;    /* Should explicit redirect to content be used (vs. internal redir) */
-
   char  remote_user_ena;
   char  show_tech;
   char  bare_url_entityid;
@@ -367,6 +368,11 @@ struct zxid_conf {
   char  md_authority_ena;
   char  backwards_compat_ena; /* Enable CBC (instead of GCM) and PKCS#1 v1.5 padding, both of which are vulnearable and can compromise modern crypto through Backwards Compatibility Attacks. */
 
+  char* xmldsig_sig_meth;
+  char* xmldsig_digest_algo;
+  char* samlsig_digest_algo;
+  char* blobsig_digest_algo;
+
 #ifdef USE_CURL
   CURL* curl;
 #endif
@@ -404,11 +410,12 @@ struct zxid_cgi {
   char  force_authn;   /* ff= Whether IdP SHOULD authenticate the user anew. */
   char  enc_hint;      /* Hint: Should NID be encrypted in SLO and MNI, see also cf->nameid_enc */
   char  atselafter;    /* at= Attribute selection requested checkbox. */
-  char  pad7;
+  char  mob;           /* Mobile device flag, detected from HTTP_USER_AGENT */
   char* sid;           /* If session is already active, the session ID. */
   char* nid;           /* NameID of the user. */
   char* uid;           /* au= Form field for user. */
   char* pw;            /* ap= Form field for password. */
+  char* pin;           /* aq= Form field for pin code (second password, used in 2 factor Yubikey. */
   char* ssoreq;        /* ar= Used for conveying original AuthnReq through authn phase. */
   char* cdc;           /* c=  Common Domain Cookie, returned by the CDC reader, also succinctID */
   char* eid;           /* e=, d= Entity ID of an IdP (typically for login) */
@@ -483,6 +490,7 @@ struct zxid_cgi {
   char* updated_time;  /* OAuth2 */
 #endif
   char* inv;           /* Invitation ID */
+  char* pcode;         /* Mobile pairing code */
   char* skin;
   char* action_url;    /* <form action=URL> in some forms, such as post.html */
   char* uri_path;      /* SCRIPT_NAME or other URI path */
@@ -708,6 +716,7 @@ struct zxid_invite {
 #define ZXID_DIMD_DIR "dimd/"
 #define ZXID_INV_DIR  "inv/"
 #define ZXID_LOG_DIR  "log/"
+#define ZXID_PCODE_DIR  "pcode/"  /* Mobile pairing codes */
 #define ZXID_DCR_DIR  "dcr/"  /* OAUTH2 Dynamic Client Registrations */
 #define ZXID_RSR_DIR  "rsr/"  /* OAUTH2 Resource Set Registrations */
 #define ZXID_MAX_USER (256)   /* Maximum size of .mni or user file */
@@ -795,10 +804,10 @@ struct zxsig_ref {
 #define ZXSIG_AUDIENCE   9  /* V Assertion has wrong audience. */
 
 #ifdef USE_OPENSSL
-ZXID_DECL struct zx_ds_Signature_s* zxsig_sign(struct zx_ctx* c, int n, struct zxsig_ref* sref, X509* cert, EVP_PKEY* priv_key);
+ZXID_DECL struct zx_ds_Signature_s* zxsig_sign(struct zx_ctx* c, int n, struct zxsig_ref* sref, X509* cert, EVP_PKEY* priv_key, const char* sig_meth_spec, const char* digest_spec);
 ZXID_DECL int zxsig_validate(struct zx_ctx* c, X509* cert, struct zx_ds_Signature_s* sig, int n, struct zxsig_ref* refs);
-ZXID_DECL int zxsig_data(struct zx_ctx* c, int len, const char* d, char** sig, EVP_PKEY* priv_key, const char* lk);
-ZXID_DECL int zxsig_verify_data(int len, char* data, int siglen, char* sig, X509* cert, char* lk);
+ZXID_DECL int zxsig_data(struct zx_ctx* c, int len, const char* d, char** sig, EVP_PKEY* priv_key, const char* lk, const char* md_alg);
+ZXID_DECL int zxsig_verify_data(int len, char* data, int siglen, char* sig, X509* cert, const char* lk, const char* mdalg);
 ZXID_DECL struct zx_xenc_EncryptedData_s* zxenc_pubkey_enc(zxid_conf* cf, struct zx_str* data, struct zx_xenc_EncryptedKey_s** ekp, X509* cert, char* idsuffix, zxid_entity* meta);
 #endif
 ZXID_DECL struct zx_str* zxenc_privkey_dec(zxid_conf* cf, struct zx_xenc_EncryptedData_s* ed, struct zx_xenc_EncryptedKey_s* ek);
@@ -872,6 +881,12 @@ ZXID_DECL EVP_PKEY*  zxid_extract_private_key(char* buf, char* name);
 ZXID_DECL X509* zxid_read_cert(zxid_conf* cf, char* name);
 ZXID_DECL EVP_PKEY*  zxid_read_private_key(zxid_conf* cf, char* name);
 ZXID_DECL int zxid_lazy_load_sign_cert_and_pkey(zxid_conf* cf, X509** cert, EVP_PKEY** pkey, const char* logkey);
+ZXID_DECL const char* zxid_get_cert_signature_algo(X509* cert);
+ZXID_DECL const char* zxsig_choose_xmldsig_sig_meth_url(EVP_PKEY* priv_key, const char* dig_alg);
+ZXID_DECL const char* zxsig_choose_xmldsig_sig_meth_urlenc(EVP_PKEY* priv_key, const char* dig_alg);
+  //ZXID_DECL const char* zxid_get_cert_signature_algo_url(X509* cert);
+  //ZXID_DECL const char* zxid_get_cert_signature_algo_urlenc(X509* cert);
+  //ZXID_DECL const char* zxid_get_cert_digest_url(X509* cert);
 #endif
 ZXID_DECL int   zxid_set_opt(zxid_conf* cf, int which, int val);
 ZXID_DECL char* zxid_set_opt_cstr(zxid_conf* cf, int which, char* val);
@@ -976,6 +991,7 @@ ZXID_DECL zxid_a7n* zxid_dec_a7n(zxid_conf* cf, zxid_a7n* a7n, struct zx_sa_Encr
 ZXID_DECL int zxid_sp_deref_art(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses);
 ZXID_DECL int zxid_as_call_ses(zxid_conf* cf, zxid_entity* idp_meta, zxid_cgi* cgi, zxid_ses* ses);
 ZXID_DECL zxid_ses* zxid_as_call(zxid_conf* cf, zxid_entity* idp_meta, const char* user, const char* pw);
+ZXID_DECL struct zx_str* zxid_start_sso_url(zxid_conf* cf, zxid_cgi* cgi);
 
 /* zxidslo */
 
@@ -1197,7 +1213,7 @@ ZXID_DECL char* zxid_get_idpnid_at_eid(zxid_conf* cf, const char* uid, const cha
  * return value of functions (which can only fail or succeed) is directly
  * used in conditional test. You will see base 0's and 1's in code.
  * Usually 1 means event was fully handled and no fall thru behaviour
- * is desired. 0 usually means the fal thru default should happen. */
+ * is desired. 0 usually means the fall thru default should happen. */
 #define ZXID_FAIL     0  /* Fall thru to default behaviour. */
 #define ZXID_OK       1  /* Don't fall thru, event fully handled. */
 #define ZXID_REDIR_OK 2  /* Don't fall thru, event fully handled. */
