@@ -55,6 +55,14 @@ int zxid_parse_cgi(zxid_conf* cf, zxid_cgi* cgi, char* qs)
   DD("qs(%s)=%p len=%d", STRNULLCHK(qs), qs, qs?strlen(qs):-1);
   if (!qs)
     return 0;
+#if 1
+  p = getenv("HTTP_USER_AGENT");
+  if (p && !memcmp(p, "Synmobile", sizeof("Synmobile")-1)) {
+    D("Mobile detected UA(%s)", p);
+    cgi->mob = 1;
+  }
+#endif
+  
   while (qs && *qs) {
     qs = zxid_qs_nv_scan(qs, &n, &v, 2);
     if (!n)
@@ -70,6 +78,7 @@ int zxid_parse_cgi(zxid_conf* cf, zxid_cgi* cgi, char* qs)
       goto unknown;
     case 'p':
       if (!strcmp(n, "prompt")) { cgi->prompt = v; break; }  /* OAUTH2 */
+      if (!strcmp(n, "pcode"))  { cgi->pcode = v; break; }   /* Mobile pairing code */
       goto unknown;
     case 'r':
       if (!strcmp(n, "response_type")) { cgi->response_type = v; break; }  /* OAUTH2/OIDC1 */
@@ -199,6 +208,7 @@ set_eid:
       case 'l': if (n[3]) goto unknown;  cgi->op = n[2];           break;  /* al = login */
       case 'u': if (n[2]) goto unknown;  if (v[0] || !cgi->uid) cgi->uid = v; break; /* au =user */
       case 'p': if (n[2]) goto unknown;  cgi->pw = v;              break;  /* ap = password */
+      case 'q': if (n[2]) goto unknown;  cgi->pin = v;             break;  /* aq = pin */
       case 'r': if (n[2]) goto unknown;  cgi->ssoreq = v;          break;  /* ar = AnRq */
       case 'n': if (n[2]) goto unknown;  cgi->op = 'N';            break;  /* an = new user */
       case 'w': if (n[2]) goto unknown;  cgi->op = 'W';            break;  /* aw = recover pw */
@@ -288,6 +298,7 @@ zxid_cgi* zxid_new_cgi(zxid_conf* cf, char* qs)
  * will be deposited in cgi->sid. If no session ID is found, then cgi->sid
  * is not modified. The name of the cookie is determined by configuration
  * option ~SES_COOKIE_NAME~ (see zxidconf.h).
+ * N.B. The session itself is not loaded, just the cgi->sid is populated.
  *
  * return:: none, but cgi->sid is modified
  *
