@@ -1,5 +1,5 @@
 /* zxidconf.c  -  Handwritten functions for parsing ZXID configuration file
- * Copyright (c) 2012-2013 Synergetics (sampo@synergetics.be), All Rights Reserved.
+ * Copyright (c) 2012-2016 Synergetics (sampo@synergetics.be), All Rights Reserved.
  * Copyright (c) 2009-2011 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.
  * Copyright (c) 2006-2009 Symlabs (symlabs@symlabs.com), All Rights Reserved.
  * Author: Sampo Kellomaki (sampo@iki.fi)
@@ -29,6 +29,10 @@
  * 20.11.2013, added %d expansion for VURL, added ECHO for debug prints --Sampo
  * 29.11.2013, added INCLUDE feature --Sampo
  * 4.12.2013,  changed URL to BURL --Sampo
+ * 11.4.2015,  added UNIX_GRP_AZ_MAP --Sampo
+ * 18.12.2015, applied patch from soconnor, perceptyx, including detection of
+ *             signature algorithm from certificate. --Sampo
+ * 8.1.2016,   added configuration options for signature and digest algorithms --Sampo
  */
 
 #include "platform.h"  /* needed on Win32 for pthread_mutex_lock() et al. */
@@ -36,11 +40,15 @@
 #include <malloc.h>
 #include <memory.h>
 #include <string.h>
+#include <stdlib.h>
+#include <sys/types.h>
+#include <grp.h>
 #ifdef USE_CURL
 #include <curl/curl.h>
 #endif
 
 #include "errmac.h"
+#include "zx.h"
 #include "zxid.h"
 #include "zxidutil.h"
 #include "zxidconf.h"
@@ -136,7 +144,7 @@ X509* zxid_extract_cert(char* buf, char* name)
 X509* zxid_read_cert(zxid_conf* cf, char* name)
 {
   X509* x = 0;  /* Forces d2i_X509() to alloc the memory. */
-  char buf[4096];
+  char buf[8192];
   char* p;
   char* e;
   p = zxid_read_cert_pem(cf, name, sizeof(buf), buf);
@@ -208,7 +216,7 @@ EVP_PKEY* zxid_extract_private_key(char* buf, char* name)
 /* Called by:  hi_new_shuffler, test_ibm_cert_problem x2, test_ibm_cert_problem_enc_dec x2, zxbus_mint_receipt x2, zxenc_privkey_dec, zxid_init_conf x3, zxid_lazy_load_sign_cert_and_pkey, zxlog_write_line x2 */
 EVP_PKEY* zxid_read_private_key(zxid_conf* cf, char* name)
 {
-  char buf[4096];
+  char buf[8192];
   int got = read_all(sizeof(buf),buf,"read_private_key",1, "%s" ZXID_PEM_DIR "%s", cf->cpath, name);
   if (!got && cf->auto_cert)
     zxid_mk_self_sig_cert(cf, sizeof(buf), buf, "read_private_key", name);
@@ -217,8 +225,27 @@ EVP_PKEY* zxid_read_private_key(zxid_conf* cf, char* name)
 
 /*() Lazy load signing certificate and private key. This reads them from disk
  * if needed. If they do not exist and auto_cert is enabled, they will be
- * generated on disk and the read. Once read from disk, they will be cached in
- * memory. */
+ * generated on disk and then read. Once read from disk, they will be cached in
+ * memory.
+ *
+ * > N.B. If the cert does not yet exist, write access to disk will be needed.
+ * > If it already exists, read access is sufficient. Thus it is more secure
+ * > to pregenerate the certificate and then set the permissions so that
+ * > the process can read it, but can not alter it.
+ *
+ * cf:: Configuration object
+ * cert:: result parameter. If non null, the certificate will be extracted
+ *     from file and pointer to the X509 data structure will be deposited
+ *     to place pointed by this parameter. If null, certificate is neither
+ *     extracted nor returned. The data structure should be freed by the
+ *     caller.
+ * pkey:: result parameter. Must be specified. The private key data structure
+ *     is extracted from the file and returned using this parameter. The
+ *     data structure should be freed by the caller.
+ * logkey:: Free form string describing why the cert and private key are
+ *     being requested. Used for logging and debugging.
+ * return:: Returns 1 on success and 0 on failure.
+ */
 
 /* Called by:  zxid_anoint_a7n, zxid_anoint_sso_resp, zxid_az_soap x3, zxid_idp_soap_dispatch x2, zxid_idp_sso, zxid_mk_art_deref, zxid_mk_at_cert, zxid_saml2_post_enc, zxid_saml2_redir_enc, zxid_sp_mni_soap, zxid_sp_slo_soap, zxid_sp_soap_dispatch x7, zxid_ssos_anreq, zxid_wsf_sign */
 int zxid_lazy_load_sign_cert_and_pkey(zxid_conf* cf, X509** cert, EVP_PKEY** pkey, const char* logkey)
@@ -235,7 +262,8 @@ int zxid_lazy_load_sign_cert_and_pkey(zxid_conf* cf, X509** cert, EVP_PKEY** pke
     return 0;
   return 1;
 }
-#endif
+
+#endif  /* USE_OPENSSL */
 
 /*() Set obscure options of ZX and ZXID layers. Used to set debug options.
  * Generally setting these options is not supported, but this function
@@ -550,6 +578,102 @@ struct zxid_map* zxid_load_map(zxid_conf* cf, struct zxid_map* map, char* v)
   return map;
 }
 
+/*() Parse unix_grp_az_map specification and add it to linked list
+ * srcns$A$rule$b$ext;src$A$rule$b$ext;...
+ * The list ends up being built in reverse order, which at runtime
+ * causes last stanzas to be evaluated first and first match is used.
+ * Thus you should place most specific rules last and most generic rules first.
+ * See also: zxid_find_map() and zxid_map_val()
+ */
+
+/* Called by:  zxid_init_conf x7, zxid_mk_usr_a7n_to_sp, zxid_parse_conf_raw x7, zxid_read_map */
+struct zxid_map* zxid_load_unix_grp_az_map(zxid_conf* cf, struct zxid_map* map, char* v)
+{
+  char* ns;
+  char* A;
+  char* val;
+  char* group;
+  char* ext;
+  char* p = v;
+  int len, n_grps, i;
+  struct zxid_map* mm;
+  struct group* grp;
+  gid_t* gids;
+
+  DD("v(%s)", v);
+
+  n_grps = getgroups(0,0);
+  gids = ZX_ALLOC(cf->ctx, (n_grps+1)*sizeof(gid_t));
+  getgroups(n_grps, gids);
+  gids[n_grps] = getegid();  /* getgroups(2) is not guaranteed to return egid */
+
+  while (p && *p) {
+    ns = p;
+    p = strchr(p, '$');
+    if (!p) {
+      ERR("Malformed UNIX_GRP_AZ_MAP directive: source namespace missing at pos %d", ((int)(p-v)));
+      return map;
+    }
+
+    A = ++p;
+    p = strchr(p, '$');
+    if (!p) {
+      ERR("Malformed UNIX_GRP_AZ_MAP directive: source attribute name missing at pos %d", ((int)(p-v)));
+      return map;
+    }
+
+    val = ++p;
+    p = strchr(p, '$');
+    if (!p) {
+      ERR("Malformed UNIX_GRP_AZ_MAP directive: value missing at pos %d", ((int)(p-v)));
+      return map;
+    }
+
+    group = ++p;
+    p = strchr(p, '$');
+    if (!p) {
+      ERR("Malformed UNIX_GRP_AZ_MAP directive: unix group name missing at pos %d", ((int)(p-v)));
+      return map;
+    }
+    
+    ext = ++p;
+    len = strcspn(p, ";\n");  /* Stanza ends in separator ; or end of string nul */
+    p = ext + len;
+    
+    mm = ZX_ZALLOC(cf->ctx, struct zxid_map);
+    mm->n = map;
+    map = mm;
+    
+    COPYVAL(mm->ns,  ns,  A-1);
+    COPYVAL(mm->src, A,   val-1);
+    COPYVAL(mm->dst, val, group-1);
+    COPYVAL(mm->ext, ext, p);
+    
+    *(ext-1) = 0;
+    grp = getgrnam(group);
+    *(ext-1) = '$';
+    if (grp) {
+      for (i = 0; i <= n_grps; ++i)
+	if (grp->gr_gid == gids[i])
+	  goto have_group;
+      ERR("UNIX_GRP_AZ_MAP: The current process does not belong to unix group name %s at pos %d (Config Error: see /etc/group for listing of groups)", group, ((int)(p-v)));
+      return map;      
+    have_group:
+      mm->rule = grp->gr_gid;
+    } else {
+      ERR("UNIX_GRP_AZ_MAP: unix group name %s does not exist at pos %d (Config Error: see /etc/group for listing of groups)", group, ((int)(p-v)));
+      return map;      
+    }
+
+    DD("map ns(%s) A(%s) val(%s) gid=%d ext(%s)", mm->ns, mm->src, mm->dst, mm->rule, mm->ext);
+    if (!*p || *p == '\n') break;
+    ++p;
+  }
+
+  ZX_FREE(cf->ctx, gids);
+  return map;
+}
+
 /*() Reverse of zxid_load_map(). */
 
 /* Called by:  zxid_free_conf x7 */
@@ -665,7 +789,7 @@ struct zxid_bus_url* zxid_load_bus_url(zxid_conf* cf, struct zxid_bus_url* bu_ro
     bu->n = bu_root;
     bu_root = bu;
     COPYVAL(bu->s, q, p);
-    COPYVAL(bu->eid, q, p);
+    COPYVAL(bu->eid, q, p);  /* *** convention is that contact URL and eid are the same?!? */
   }
   return bu_root;
 }
@@ -896,6 +1020,52 @@ struct zxid_attr* zxid_find_at(struct zxid_attr* pool, const char* name)
   return 0;
 }
 
+/*() Check that the user, who is logged into session, maps to group.
+ * This is used by UNIX_GRP_AZ_MAP to check that filesystem
+ * permissions allow user to access a file (existence of g+r and
+ * user mapping to the correct group).
+ *
+ * return:: 0=deny, 1=permit */
+
+int zxid_unix_grp_az_check(zxid_conf* cf, zxid_ses* ses, int gid)
+{
+  struct zxid_map* grp_map = 0;
+  struct zxid_attr* at = 0;
+  
+  if (!cf || !ses) {
+    ERR("missing argument cf=%p", cf);
+    return 0;
+  }
+  if (!ses->nid || !ses->nid[0]) {
+    INFO("user not logged in ses->nid=%p", ses->nid);
+    return 0;
+  }
+  for (grp_map = cf->unix_grp_az_map; grp_map; grp_map = grp_map->n) {
+    if (grp_map->rule != gid)
+      continue;
+
+    /* If affiliation filter is specified, check it. */
+    if (grp_map->ns && strcmp(grp_map->ns, "") /* none of the wild card cases */
+	&& strcmp(grp_map->ns, "*") && strcmp(grp_map->ns, "**")) {
+      at = zxid_find_at(ses->at, "affid");
+      if (!at || !zx_match(grp_map->ns, at->val /*ses->nameid->NameQualifier*/))
+	continue;
+    }
+
+    /* If attribute filter is specified, check it. */
+    if (grp_map->src && strcmp(grp_map->src, "") /* none of the wild card cases */
+	&& strcmp(grp_map->src, "*") && strcmp(grp_map->src, "**")) {
+      at = zxid_find_at(ses->at, grp_map->src);
+      if (!at || !zx_match(grp_map->dst, at->val /*ses->nameid->NameQualifier*/))
+	continue;
+    }
+    D("%s=%s maps to gid=%d", STRNULLCHKD(grp_map?grp_map->src:0), STRNULLCHKD(at?at->val:0), gid);
+    return 1;
+  }
+  INFO("user does not map to gid=%d", gid);
+  return 0;
+}
+
 /*() Given URL, return a newly allocated string corresponding
  * to the domain name part of the URL. Used to grab fedusername_suffix
  * from the url config option. */
@@ -972,6 +1142,7 @@ int zxid_init_conf(zxid_conf* cf, const char* zxid_path)
   cf->wsp_pat       = ZXID_WSP_PAT;
   cf->uma_pat       = ZXID_UMA_PAT;
   cf->sso_pat       = ZXID_SSO_PAT;
+  cf->cdc_url       = ZXID_CDC_URL;
   cf->cdc_choice    = ZXID_CDC_CHOICE;
   cf->authn_req_sign = ZXID_AUTHN_REQ_SIGN;
   cf->want_sso_a7n_signed = ZXID_WANT_SSO_A7N_SIGNED;
@@ -1063,7 +1234,7 @@ int zxid_init_conf(zxid_conf* cf, const char* zxid_path)
   cf->anon_ok        = ZXID_ANON_OK;
   cf->optional_login_pat = ZXID_OPTIONAL_LOGIN_PAT;
   cf->required_authnctx = ZXID_REQUIRED_AUTHNCTX;	/* NB: NULL. */
-  cf->issue_authnctx_pw = ZXID_ISSUE_AUTHNCTX_PW;
+  cf->issue_authnctx = zxid_load_cstr_list(cf, 0, ZXID_ISSUE_AUTHNCTX);
   cf->idp_pref_acs_binding = ZXID_IDP_PREF_ACS_BINDING;
   cf->mandatory_attr = ZXID_MANDATORY_ATTR;
 
@@ -1097,6 +1268,8 @@ int zxid_init_conf(zxid_conf* cf, const char* zxid_path)
   cf->wsp_localpdp_obl_emit   = ZXID_WSP_LOCALPDP_OBL_EMIT;
   cf->wsc_localpdp_obl_accept = ZXID_WSC_LOCALPDP_OBL_ACCEPT;
 
+  cf->unix_grp_az_map   = zxid_load_unix_grp_az_map(cf, 0, ZXID_UNIX_GRP_AZ_MAP);
+
   cf->redir_to_content  = ZXID_REDIR_TO_CONTENT;
   cf->remote_user_ena   = ZXID_REMOTE_USER_ENA;
   cf->max_soap_retry    = ZXID_MAX_SOAP_RETRY;
@@ -1144,6 +1317,11 @@ int zxid_init_conf(zxid_conf* cf, const char* zxid_path)
   cf->mgmt_footer       = ZXID_MGMT_FOOTER;
   cf->mgmt_end          = ZXID_MGMT_END;
   
+  cf->xmldsig_sig_meth  = ZXID_XMLDSIG_SIG_METH;
+  cf->xmldsig_digest_algo = ZXID_XMLDSIG_DIGEST_ALGO;
+  cf->samlsig_digest_algo = ZXID_SAMLSIG_DIGEST_ALGO;
+  cf->blobsig_digest_algo = ZXID_BLOBSIG_DIGEST_ALGO;
+
   LOCK_INIT(cf->mx);
   LOCK_INIT(cf->curl_mx);
   if (!zxid_ent_cache_mx_init) {
@@ -1181,6 +1359,8 @@ void zxid_free_conf(zxid_conf *cf)
   zxid_free_cstr_list(cf, cf->localpdp_role_deny);
   zxid_free_cstr_list(cf, cf->localpdp_idpnid_permit);
   zxid_free_cstr_list(cf, cf->localpdp_idpnid_deny);
+  zxid_free_cstr_list(cf, cf->issue_authnctx);
+  zxid_free_map(cf, cf->unix_grp_az_map);
   if (cf->required_authnctx) {
     ZX_FREE(cf->ctx, cf->required_authnctx);
   }
@@ -1494,7 +1674,7 @@ static int zxid_parse_vpath(zxid_conf* cf, char* vpath)
   
   if (*vpath != '/') {
     if (cf->cpath_len > lim-np) {
-      ERR("TOO LONG: PATH(%.*s) len=%d does not fit in vpath buffer size=%ld", cf->cpath_len, cf->cpath, cf->cpath_len, (long)(lim-np));
+      ERR("TOO LONG: CPATH(%.*s) len=%d does not fit in vpath buffer size=%ld", cf->cpath_len, cf->cpath, cf->cpath_len, (long)(lim-np));
       return 0;
     }
     memcpy(np, cf->cpath, cf->cpath_len);
@@ -1503,7 +1683,7 @@ static int zxid_parse_vpath(zxid_conf* cf, char* vpath)
   
   zxid_expand_percent(vpath, np, lim, 0);
   if (--zxid_suppress_vpath_warning > 0) {
-    INFO("VPATH(%s) alters PATH(%s) to new PATH(%s)", vpath, cf->cpath, newpath);
+    INFO("VPATH(%s) alters CPATH(%s) to new CPATH(%s)", vpath, cf->cpath, newpath);
   }
   zxid_parse_conf_path_raw(cf, zx_dup_cstr(cf->ctx, newpath), 1);
   return 1;
@@ -1517,7 +1697,7 @@ static int zxid_parse_vurl(zxid_conf* cf, char* vurl)
   char newurl[PATH_MAX];
   zxid_expand_percent(vurl, newurl, newurl + sizeof(newurl), 1);
   if (--zxid_suppress_vpath_warning > 0) {
-    INFO("VURL(%s) alters URL(%s) to new URL(%s)", vurl, cf->burl, newurl);
+    INFO("VURL(%s) alters BURL(%s) to new BURL(%s)", vurl, cf->burl, newurl);
   }
   cf->burl = zx_dup_cstr(cf->ctx, newurl);
   return 1;
@@ -1600,6 +1780,7 @@ int zxid_parse_conf_raw(zxid_conf* cf, int qs_len, char* qs)
       if (!strcmp(n, "BUS_URL"))         { cf->bus_url = zxid_load_bus_url(cf, cf->bus_url, v);   break; }
       if (!strcmp(n, "BUS_PW"))          { cf->bus_pw = v; break; }
       if (!strcmp(n, "BACKWARDS_COMPAT_ENA")) { SCAN_INT(v, cf->backwards_compat_ena); break; }
+      if (!strcmp(n, "BLOBSIG_DIGEST_ALGO")) { cf->blobsig_digest_algo = v; break; }
       goto badcf;
     case 'C':  /* CDC_URL, CDC_CHOICE */
       if (!strcmp(n, "CPATH"))           goto path;
@@ -1641,7 +1822,7 @@ int zxid_parse_conf_raw(zxid_conf* cf, int qs_len, char* qs)
     case 'I':  /* ISSUE_A7N, ISSUE_MSG */
       if (!strcmp(n, "ISSUE_A7N"))       { SCAN_INT(v, cf->log_issue_a7n); break; }
       if (!strcmp(n, "ISSUE_MSG"))       { SCAN_INT(v, cf->log_issue_msg); break; }
-      if (!strcmp(n, "ISSUE_AUTHNCTX_PW")) { cf->issue_authnctx_pw = v; break; }
+      if (!strcmp(n, "ISSUE_AUTHNCTX"))  { cf->issue_authnctx = zxid_load_cstr_list(cf, cf->issue_authnctx, v); break; }
 #if 0
       if (!strcmp(n, "IDP_SEL_START"))   { cf->idp_sel_start = v; break; }
       if (!strcmp(n, "IDP_SEL_NEW_IDP")) { cf->idp_sel_new_idp = v; break; }
@@ -1795,6 +1976,7 @@ int zxid_parse_conf_raw(zxid_conf* cf, int qs_len, char* qs)
       if (!strcmp(n, "STATE"))          { cf->state = v; break; }
       if (!strcmp(n, "SSO_PAT"))        { cf->sso_pat = v; break; }
       if (!strcmp(n, "SOAP_ACTION_HDR")) { cf->soap_action_hdr = v; break; }
+      if (!strcmp(n, "SAMLSIG_DIGEST_ALGO")) { cf->samlsig_digest_algo = v; break; }
       goto badcf;
     case 'T':  /* TIMEOUT_FATAL */
       if (!strcmp(n, "TIMEOUT_FATAL"))  { SCAN_INT(v, cf->timeout_fatal); break; }
@@ -1805,6 +1987,7 @@ int zxid_parse_conf_raw(zxid_conf* cf, int qs_len, char* qs)
       if (!strcmp(n, "URL"))            { cf->burl = v; cf->fedusername_suffix = zxid_grab_domain_name(cf, cf->burl); break; }
       if (!strcmp(n, "USER_LOCAL"))     { SCAN_INT(v, cf->user_local); break; }
       if (!strcmp(n, "UMA_PAT"))        { cf->uma_pat = v; break; }
+      if (!strcmp(n, "UNIX_GRP_AZ_MAP")) { cf->unix_grp_az_map = zxid_load_unix_grp_az_map(cf, cf->unix_grp_az_map, v); break; }
       goto badcf;
     case 'V':  /* VALID_OPT */
       if (!strcmp(n, "VALID_OPT"))      { SCAN_INT(v, cf->valid_opt); break; }
@@ -1833,6 +2016,8 @@ int zxid_parse_conf_raw(zxid_conf* cf, int qs_len, char* qs)
       goto badcf;
     case 'X':  /* XASP_VERS */
       if (!strcmp(n, "XASP_VERS"))      { cf->xasp_vers = v; break; }
+      if (!strcmp(n, "XMLDSIG_SIG_METH")) { cf->xmldsig_sig_meth = v; break; }
+      if (!strcmp(n, "XMLDSIG_DIGEST_ALGO")) { cf->xmldsig_digest_algo = v; break; }
       goto badcf;
     default:
     badcf:
@@ -1961,6 +2146,8 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
   struct zx_str* localpdp_role_deny;
   struct zx_str* localpdp_idpnid_permit;
   struct zx_str* localpdp_idpnid_deny;
+  struct zx_str* issue_authnctx;
+  struct zx_str* unix_grp_az_map;
   if (cf->log_level>0)
     zxlog(cf, 0, 0, 0, 0, 0, 0, 0, "N", "W", "MYCONF", 0, 0);
 
@@ -2016,6 +2203,10 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
   localpdp_idpnid_permit = zxid_show_cstr_list(cf, cf->localpdp_idpnid_permit);
   localpdp_idpnid_deny   = zxid_show_cstr_list(cf, cf->localpdp_idpnid_deny);
 
+  issue_authnctx = zxid_show_cstr_list(cf, cf->issue_authnctx);
+
+  unix_grp_az_map = zxid_show_map(cf, cf->unix_grp_az_map);
+
   eid = zxid_my_ent_id_cstr(cf);
 
   return zx_strf(cf->ctx,
@@ -2025,6 +2216,7 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 "<p>Version: R" ZXID_REL " (" ZXID_COMPILE_DATE ")\n"
 
 "<pre>"
+"DEBUG=0x%x\n"
 "CPATH=%s\n"
 "BURL=%s\n"
 "AFFILIATION=%s\n"
@@ -2040,7 +2232,7 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 "CONTACT_EMAIL=%s\n"
 "CONTACT_TEL=%s\n"
 "FEDUSERNAME_SUFFIX=%s\n"
-"#ZXID_CONF_FILE=%d (compile)\n"
+"#ZXID_CONF_FILE_ENA=%d (compile)\n"
 "#ZXID_CONF_FLAG=%d (compile)\n"
 "NON_STANDARD_ENTITYID=%s\n"
 "REDIRECT_HACK_IMPOSED_URL=%s\n"
@@ -2142,7 +2334,7 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 
 "ANON_OK=%s\n"
 "OPTIONAL_LOGIN_PAT=%s\n"
-"ISSUE_AUTHNCTX_PW=%s\n"
+"ISSUE_AUTHNCTX=%s\n"
 "IDP_PREF_ACS_BINDING=%s\n"
 "MANDATORY_ATTR=%s\n"
 "PDP_URL=%s\n"
@@ -2154,6 +2346,11 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 "SHOW_TECH=%d\n"
 "WD=%s\n"
 
+"XMLDSIG_SIG_METH=%s\n"
+"XMLDSIG_DIGEST_ALGO=%s\n"
+"SAMLSIG_DIGEST_ALGO=%s\n"
+"BLOBSIG_DIGEST_ALGO=%s\n"
+
 "IDP_LIST_METH=%d\n"
 "IDP_SEL_PAGE=%s\n"
 "IDP_SEL_TEMPL_FILE=%s\n"
@@ -2225,10 +2422,12 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 //"WSP_LOCALPDP_OBL_REQ=%s\n"
 "WSP_LOCALPDP_OBL_EMIT=%s\n"
 //"WSC_LOCALPDP_OBL_ACCEPT=%s\n"
+"UNIX_GRP_AZ_MAP=\n%s\n"
 "</pre>",
 		 cf->burl, eid,
 		 cf->cpath,
 
+		 errmac_debug,
 		 cf->cpath,
 		 cf->burl,
 		 STRNULLCHK(cf->affiliation),
@@ -2244,7 +2443,7 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 		 STRNULLCHK(cf->contact_email),
 		 STRNULLCHK(cf->contact_tel),
 		 STRNULLCHK(cf->fedusername_suffix),
-		 ZXID_CONF_FILE,
+		 ZXID_CONF_FILE_ENA,
 		 ZXID_CONF_FLAG,
 		 STRNULLCHK(cf->non_standard_entityid),
 		 STRNULLCHK(cf->redirect_hack_imposed_url),
@@ -2254,12 +2453,12 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 		 STRNULLCHK(cf->wsp_pat),
 		 STRNULLCHK(cf->uma_pat),
 		 STRNULLCHK(cf->sso_pat),
-		 STRNULLCHK(cf->cdc_url),
 		 STRNULLCHK(cf->wsc_soap_content_type),
 		 STRNULLCHK(cf->wsc_to_hdr),
 		 STRNULLCHK(cf->wsc_replyto_hdr),
 		 STRNULLCHK(cf->wsc_action_hdr),
 		 STRNULLCHK(cf->soap_action_hdr),
+		 STRNULLCHK(cf->cdc_url),
 		 cf->cdc_choice,
 
 		 STRNULLCHK(cf->load_cot_cache),
@@ -2345,7 +2544,7 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 
 		 STRNULLCHK(cf->anon_ok),
 		 STRNULLCHK(cf->optional_login_pat),
-		 STRNULLCHK(cf->issue_authnctx_pw),
+		 issue_authnctx->s,
 		 STRNULLCHK(cf->idp_pref_acs_binding),
 		 STRNULLCHK(cf->mandatory_attr),
 		 STRNULLCHK(cf->pdp_url),
@@ -2356,6 +2555,10 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 		 cf->bare_url_entityid,
 		 cf->show_tech,
 		 STRNULLCHK(cf->wd),
+		 STRNULLCHK(cf->xmldsig_sig_meth),
+		 STRNULLCHK(cf->xmldsig_digest_algo),
+		 STRNULLCHK(cf->samlsig_digest_algo),
+		 STRNULLCHK(cf->blobsig_digest_algo),
 
 		 cf->idp_list_meth,
 		 STRNULLCHK(cf->idp_sel_page),
@@ -2413,8 +2616,9 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 		 localpdp_idpnid_deny->s,
 		 STRNULLCHK(cf->wsc_localpdp_obl_pledge),
 		 //STRNULLCHK(cf->wsp_localpdp_obl_req),
-		 STRNULLCHK(cf->wsp_localpdp_obl_emit) //,
+		 STRNULLCHK(cf->wsp_localpdp_obl_emit),
 		 //STRNULLCHK(cf->wsc_localpdp_obl_accept)
+		 unix_grp_az_map->s //,
 	 );
 }
 
