@@ -92,7 +92,7 @@ struct zx_sp_Response_s* zxid_ssos_anreq(zxid_conf* cf, zxid_ses* ses, struct zx
     refs.id = &outa7n->ID->g;
     refs.canon = zx_easy_enc_elem_sig(cf, &outa7n->gg);
     if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert paos")) {
-      outa7n->Signature = zxsig_sign(cf->ctx, 1, &refs, sign_cert, sign_pkey);
+      outa7n->Signature = zxsig_sign(cf->ctx, 1, &refs, sign_cert, sign_pkey, cf->xmldsig_sig_meth, cf->xmldsig_digest_algo);
       zx_add_kid_after_sa_Issuer(&outa7n->gg, &outa7n->Signature->gg);
     }
   }
@@ -171,7 +171,7 @@ zxid_tok* zxid_map_identity_token(zxid_conf* cf, zxid_ses* ses, const char* at_e
   inp->TokenPolicy->NameIDPolicy->AllowCreate = zx_ref_attr(cf->ctx, &inp->TokenPolicy->NameIDPolicy->gg, zx_AllowCreate_ATTR, XML_TRUE); /* default false */
 
   env = zxid_wsc_call(cf, ses, epr, env, 0);
-  if (!env || !env->Body) {
+  if (!env || env == (void*)ZXID_REDIR_OK || !env->Body) {
     ERR("Identity Mapping call failed envelope=%p", env);
     return 0;
   }
@@ -400,7 +400,7 @@ zxid_tok* zxid_nidmap_identity_token(zxid_conf* cf, zxid_ses* ses, const char* a
   req->NameID = ses->nameid;  /* or tgtnameid? */
 
   env = zxid_wsc_call(cf, ses, epr, env, 0);
-  if (!env || !env->Body) {
+  if (!env || env == (void*)ZXID_REDIR_OK || !env->Body) {
     ERR("Identity Mapping call failed envelope=%p", env);
     return 0;
   }
