@@ -1,5 +1,5 @@
 /* zxidsimp.c  -  Handwritten zxid_simple() API
- * Copyright (c) 2012-2014 Synergetics NV (sampo@synergetics.be), All Rights Reserved.
+ * Copyright (c) 2012-2016 Synergetics NV (sampo@synergetics.be), All Rights Reserved.
  * Copyright (c) 2009-2011 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.
  * Copyright (c) 2007-2009 Symlabs (symlabs@symlabs.com), All Rights Reserved.
  * Author: Sampo Kellomaki (sampo@iki.fi)
@@ -25,6 +25,7 @@
  * 17.11.2013, move redir_to_content feature to zxid_simple() --Sampo
  * 20.11.2013, move defaultqs feature feature to zxid_simple() --Sampo
  * 14.2.2014,  added redirafter feature for local IdP logins (e.g. zxidatsel.pl) --Sampo
+ * 1.4.2015,   fixed skin based template path in case it does not have directory --Sampo
  *
  * Login button abbreviations
  * A2 = SAML 2.0 Artifact Profile
@@ -111,7 +112,7 @@ int zxid_conf_to_cf_len(zxid_conf* cf, int conf_len, const char* conf)
     }
 
     buf = getenv(ZXID_ENV_PREFIX "PRE_CONF");
-    D("Check " ZXID_ENV_PREFIX "PRE_CONF(%s)", buf);
+    D("Check " ZXID_ENV_PREFIX "PRE_CONF(%s)", STRNULLCHKD(buf));
     if (buf) {
       /* Copy the conf string because we are going to modify it in place. */
       D("Applying " ZXID_ENV_PREFIX "PRE_CONF(%s)", buf);
@@ -301,6 +302,9 @@ static const char* zxid_map_bangbang(zxid_conf* cf, zxid_cgi* cgi, const char* k
   case 'A':
     if (BBMATCH("ACTION_URL", key, lim)) return cgi->action_url;
     break;
+  case 'B':
+    if (BBMATCH("BURL", key, lim)) return cf->burl;
+    break;
   case 'D':
     if (BBMATCH("DBG", key, lim)) return cgi->dbg;
     break;
@@ -380,10 +384,12 @@ struct zx_str* zxid_template_page_cf(zxid_conf* cf, zxid_cgi* cgi, const char* t
     for (p = templ_path + strlen(templ_path)-1;
 	 p >= templ_path && !ONE_OF_2(*p, '/', '\\');
 	 --p);
-
-    templ = read_all_alloc(cf->ctx, "templ", 1, 0, "%.*s/%s%s",
-			   p-templ_path, templ_path, cgi->skin, p);
-    D("Tried to read from skin(%s) %p", cgi->skin, templ);
+    if (p < templ_path)  /* there was no directory component */
+      templ = read_all_alloc(cf->ctx, "templ", 1, 0, "%s/%s", cgi->skin, templ_path);
+    else
+      templ = read_all_alloc(cf->ctx, "templ", 1, 0, "%.*s/%s%s",
+			     p-templ_path, templ_path, cgi->skin, p);
+    D("Tried to read from skin(%s) templ_path(%s) %p", cgi->skin, templ_path, templ);
   }
   
   if (!templ)
@@ -1047,9 +1053,8 @@ static char* zxid_simple_idp_show_an(zxid_conf* cf, zxid_cgi* cgi, int* res_len,
   zxid_ses sess;
   ZERO(&sess, sizeof(sess));
   D("cf=%p cgi=%p", cf, cgi);
-  
   DD("z saml_req(%s) rs(%s) sigalg(%s) sig(%s)", cgi->saml_req, cgi->rs, cgi->sigalg, cgi->sig);  
-  if (cgi->uid && zxid_pw_authn(cf, cgi, &sess)) {  /* Try login, just in case. */
+  if ((cgi->uid || cgi->pcode) && zxid_pw_authn(cf, cgi, &sess)) {  /* Try login, just in case. */
     return zxid_simple_idp_an_ok_do_rest(cf, cgi, &sess, res_len, auto_flags);
   }
   if (cgi->redirafter) { /* Save next screen for local login (e.g. zxidatsel.pl */
@@ -1168,6 +1173,17 @@ static char* zxid_simple_idp_show_an(zxid_conf* cf, zxid_cgi* cgi, int* res_len,
 	*p = '_';
       }
   }
+#if 1
+  /* Hack: Different page for mobile */
+  if (cgi->mob) {
+    D("Mobile detected TF(%s)", cf->an_templ_file);
+    /* Replace final .html  with -mob.html */
+    cf->an_templ_file = zx_alloc_sprintf(cf->ctx, 0, "%.*s-mob.html",
+					 strlen(cf->an_templ_file)-sizeof(".html")+1,
+					 cf->an_templ_file);
+    D("New TF(%s)", cf->an_templ_file);
+  }
+#endif
   ss = zxid_template_page_cf(cf, cgi, cf->an_templ_file, cf->an_templ, 4096, auto_flags);
   /* if (cgi->ssoreq) ZX_FREE(cf->ctx, cgi->ssoreq); might not be malloc'd if tabs have CGI */
   DD("an_page: ret(%s)", ss?ss->len:1, ss?ss->s:"?");
@@ -1212,7 +1228,7 @@ static char* zxid_simple_idp_new_user(zxid_conf* cf, zxid_cgi* cgi, int* res_len
 
   if (cf->new_user_page && cf->new_user_page[0]) {
     p = zx_alloc_sprintf(cf->ctx, 0, "ar=%s&zxrfr=F%s%s%s%s&zxidpurl=%s",
-		 cgi->ssoreq,
+		 STRNULLCHK(cgi->ssoreq),
 		 cgi->zxapp && cgi->zxapp[0] ? "&zxapp=" : "", cgi->zxapp ? cgi->zxapp : "",
 		 cgi->err && cgi->err[0] ? "&err=" : "", cgi->err ? cgi->err : "",
 		 cf->burl);
@@ -1238,7 +1254,7 @@ static char* zxid_simple_idp_recover_password(zxid_conf* cf, zxid_cgi* cgi, int*
 
   if (cf->recover_passwd && cf->recover_passwd[0]) {
     p = zx_alloc_sprintf(cf->ctx, 0, "ar=%s&zxrfr=F%s%s%s%s&zxidpurl=%s",
-		 cgi->ssoreq,
+		 STRNULLCHK(cgi->ssoreq),
 		 cgi->zxapp && cgi->zxapp[0] ? "&zxapp=" : "", cgi->zxapp ? cgi->zxapp : "",
 		 cgi->err && cgi->err[0] ? "&err=" : "", cgi->err ? cgi->err : "",
 		 cf->burl);
@@ -1265,9 +1281,10 @@ static char* zxid_show_protected_content_setcookie(zxid_conf* cf, zxid_cgi* cgi,
   char* rs_qs;
 
   if (cf->ses_cookie_name && *cf->ses_cookie_name) {
-    ses->setcookie = zx_alloc_sprintf(cf->ctx, 0, "%s=%s; path=/%s",
+    ses->setcookie = zx_alloc_sprintf(cf->ctx, 0, "%s=%s; path=/%s%s",
 				      cf->ses_cookie_name, ses->sid,
-				      ONE_OF_2(cf->burl[4], 's', 'S')?"; secure":"");
+				      cgi->mob?"; Max-Age=15481800":"",
+				      ONE_OF_2(cf->burl[4], 's', 'S')?"; secure; HttpOnly":"; HttpOnly");
     ses->cookie = zx_alloc_sprintf(cf->ctx, 0, "$Version=1; %s=%s",
 				   cf->ses_cookie_name, ses->sid);
     D("setcookie(%s)=(%s) ses=%p", cf->ses_cookie_name, ses->setcookie, ses);
@@ -1377,7 +1394,7 @@ char* zxid_simple_ses_active_cf(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, int
    *
    * M = CDC redirect and LECP detect
    * C = CDC reader
-   * E = Normal "Entry" page (e.g. after CDC read)
+   * E = Normal "Entry" page (e.g. after CDC read, idpsel)
    * L = Start SSO (submit of E)
    * A = Artifact processing
    * N = New User, during IdP Login (form an)
@@ -1736,10 +1753,14 @@ char* zxid_simple_cf_ses(zxid_conf* cf, int qs_len, char* qs, zxid_ses* ses, int
     }
   }
   UNLOCK(cf->mx, "simple ipport");
+
+  cgi.uri_path = getenv("SCRIPT_NAME");
+  cgi.qs = qs;  /* save orig for use in zxid_sso_set_relay_state_to_return_to_this_url() */
   
   if (!qs) {
-    qs = getenv("QUERY_STRING");
+    cgi.qs = qs = getenv("QUERY_STRING");
     if (qs) {
+      qs = zx_dup_cstr(cf->ctx, qs);
       D("QUERY_STRING(%s) %s %d", STRNULLCHK(qs), ZXID_REL, errmac_debug);
       zxid_parse_cgi(cf, &cgi, qs);
       if (ONE_OF_8(cgi.op, 'H', 'J', 'P', 'R', 'S', 'T', 'Y', 'Z')) {
@@ -1814,15 +1835,14 @@ char* zxid_simple_cf_ses(zxid_conf* cf, int qs_len, char* qs, zxid_ses* ses, int
       exit(1);
     }
     D("QUERY_STRING(%s) %s", STRNULLCHK(qs), ZXID_REL);
+    if (qs)
+      qs = zx_dup_cstr(cf->ctx, qs);
     zxid_parse_cgi(cf, &cgi, qs);
   }
   
   if (!cgi.op && !cf->bare_url_entityid)
     cgi.op = 'M';  /* By default, if no ses, check CDC and offer SSO */
   
-  cgi.uri_path = getenv("SCRIPT_NAME");
-  cgi.qs = qs;
-
   if (!cgi.sid && cf->ses_cookie_name && *cf->ses_cookie_name)
     zxid_get_sid_from_cookie(cf, &cgi, getenv("HTTP_COOKIE"));
 
@@ -1838,6 +1858,8 @@ char* zxid_simple_cf_ses(zxid_conf* cf, int qs_len, char* qs, zxid_ses* ses, int
   res = zxid_simple_no_ses_cf(cf, &cgi, ses, res_len, auto_flags);
 
 done:
+  if (qs)
+    ZX_FREE(cf->ctx, qs);
   if (buf)
     ZX_FREE(cf->ctx, buf);
   return res;
