@@ -47,7 +47,7 @@
 
 /*() This function makes the policy decision about which profile to
  * use. It is only used if there was no explicit specification in the
- * CGI form (e.g. "Login (P)" button. Currently it is a stub that
+ * CGI form (e.g. "Login (P)" button). Currently it is a stub that
  * always picks the SAML artifact profile. Eventually configuration options
  * or cgi input can be used to determine the profile in a more
  * sophisticated way. Often zxid_mk_authn_req() will override the
@@ -155,7 +155,7 @@ char* zxid_saml2_map_authn_ctx(char* c)
 
 /*() cgi->rs will be copied to ses->rs and from there in ab_pep to resource-id.
  * We compress and safe_base64 encode it to protect any URL special characters. */
-
+/* Called by:  zxid_start_sso_url, zxid_simple_show_idp_sel */
 void zxid_sso_set_relay_state_to_return_to_this_url(zxid_conf* cf, zxid_cgi* cgi)
 {
   struct zx_str* ss;
@@ -178,7 +178,8 @@ void zxid_sso_set_relay_state_to_return_to_this_url(zxid_conf* cf, zxid_cgi* cgi
  *
  * cf::     Used for many configuration options and memory allocation
  * cgi::    Used to pick the desired SSO profile based on hidden fields or user
- *     input. The cgi->rs filed specifies the URL to redirect to after the SSO.
+ *     input. The cgi->rs field specifies the URL to redirect to after the SSO.
+ *     The cgi->eid specifies the IdP entity ID.
  * return:: Redirect URL as zx_str. Caller should eventually free this memory.
  */
 /* Called by:  zxid_start_sso_location */
@@ -193,8 +194,8 @@ struct zx_str* zxid_start_sso_url(zxid_conf* cf, zxid_cgi* cgi)
   D_INDENT("start_sso: ");
   D("cgi=%p cgi->eid=%p eid(%s) pr_ix=%d", cgi, cgi->eid, STRNULLCHKD(cgi->eid), cgi->pr_ix);
   zxid_sso_set_relay_state_to_return_to_this_url(cf, cgi);
-  if (!cgi->pr_ix || !cgi->eid || !cgi->eid[0]) {
-    D("Either protocol index or entity ID missing %d", cgi->pr_ix);
+  if (!cgi->eid || !cgi->eid[0]) {
+    D("Entity ID missing %p", cgi->eid);
     cgi->err = "IdP URL Missing or incorrect";
     D_DEDENT("start_sso: ");
     return 0;
@@ -766,7 +767,7 @@ int zxid_sp_sso_finalize(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, zxid_a7n*
   DD("Creating session... %d", 0);
   ses->ssores = 0;
   zxid_put_ses(cf, ses);
-  zxid_snarf_eprs_from_ses(cf, ses);  /* Harvest attributes and bootstrap(s) */
+  zxid_snarf_eprs_from_ses(cf, ses);  /* Harvest bootstrap(s) from attribute statements */
   cgi->msg = "SSO completed and session created.";
   cgi->op = '-';  /* Make sure management screen does not try to redispatch. */
   zxid_put_user(cf, &ses->nameid->Format->g, &ses->nameid->NameQualifier->g, &ses->nameid->SPNameQualifier->g, ZX_GET_CONTENT(ses->nameid), 0);
