@@ -194,6 +194,41 @@ int zxid_put_user(zxid_conf* cf, struct zx_str* nidfmt, struct zx_str* idpent, s
 
 static char* login_failed = "Login failed. Check username and password. Make sure you have an active local account. Or just try some other authentication method or another IdP.<p>";
 
+/*() Authenticate by a pairing code.
+ * Pairing code is generated by an external program such as idppairing.pl
+ * after user has logged in. Pairing code can be introduced in another device,
+ * such as mobile phone with limited keyboard, to authenticate the user based
+ * on original authentication to the pairing web site.
+ */
+
+static int zxid_check_mobile_pairing(zxid_conf* cf, zxid_cgi* cgi)
+{
+  int len, secs;
+  char* uid;
+  char buf[1024];
+  len = read_all(sizeof(buf), (char*)buf, "pairing", 0, "%s" ZXID_PCODE_DIR "%s", cf->cpath, cgi->pcode);
+  if (len <= 0) {
+    ERR("Bad pairing pcode(%s)", cgi->pcode);
+    return 0;
+  }
+  secs = atoi(buf);
+  if (secs < time(0)) {
+    ERR("Bad pairing pcode(%s) buf(%s) expired=%d, now=%d", cgi->pcode, buf, secs, (int)time(0));
+    return 0;
+  }
+  uid = strchr(buf, ' ');
+  if (!uid) {
+    ERR("Bad pairing pcode(%s) buf(%s) uid not found", cgi->pcode, buf);
+    return 0;
+  }
+  ++uid;
+  D("Pairing OK pcode(%s) buf(%s) expired=%d, now=%d uid=%s", cgi->pcode, buf, secs, (int)time(0), uid);
+  cgi->uid = zx_dup_cstr(cf->ctx, uid);
+  snprintf(buf, sizeof(buf), "%s" ZXID_PCODE_DIR "%s", cf->cpath, cgi->pcode);
+  unlink(buf);
+  return 2;  /* *** what is good authentication context class for pairing? Password equivalent? */
+}
+
 /*() Locally authenticate user. If successful, create a session.
  * Expects to get username and password in cgi->au and cgi->ap
  * respectively. User authetication is done against local database or
@@ -207,9 +242,16 @@ static char* login_failed = "Login failed. Check username and password. Make sur
 /* Called by:  zxid_idp_as_do, zxid_simple_idp_pw_authn, zxid_simple_idp_show_an */
 int zxid_pw_authn(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses)
 {
+  int an_level;
   struct zx_str* ss;
+  struct zxid_cstr_list* ac;
 
-  if (!zx_password_authn(cf->cpath, cgi->uid, cgi->pw, 0)) {
+  if (cgi->pcode) {
+    an_level = zxid_check_mobile_pairing(cf, cgi);
+  } else {
+    an_level = zx_password_authn(cf->cpath, cgi->uid, cgi->pw, cgi->pin, 0);
+  }
+  if (!an_level) {
     cgi->err = login_failed;
     return 0;
   }
@@ -219,7 +261,12 @@ int zxid_pw_authn(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses)
   ZERO(ses, sizeof(zxid_ses));
   ses->magic = ZXID_SES_MAGIC;
   ses->an_instant = time(0);  /* This will be later used by AuthnStatement constructor. */
-  ses->an_ctx = cf->issue_authnctx_pw;  /* *** Should also depend on how user was registered */
+  
+  for (ac = cf->issue_authnctx; ac && an_level > 0; ac = ac->n, --an_level) ;
+  if (!ac)
+    ac = cf->issue_authnctx;
+  ses->an_ctx = ac->s;
+  
   /* Master session. Each pairwise SSO has its own to avoid correlation, see zxid_mk_an_stmt() */
   ss = zxid_mk_id(cf, "MMSES", ZXID_ID_BITS);
   ses->sesix = ss->s;
@@ -228,9 +275,10 @@ int zxid_pw_authn(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses)
   ses->uid = cgi->uid;
   zxid_put_ses(cf, ses);
   if (cf->ses_cookie_name && *cf->ses_cookie_name) {
-    ses->setcookie = zx_alloc_sprintf(cf->ctx, 0, "%s=%s; path=/%s",
+    ses->setcookie = zx_alloc_sprintf(cf->ctx, 0, "%s=%s; path=/%s%s",
 				      cf->ses_cookie_name, ses->sid,
-				      ONE_OF_2(cf->burl[4], 's', 'S')?"; secure":"");
+				      cgi->mob?"; Max-Age=15481800":"",
+				      ONE_OF_2(cf->burl[4], 's', 'S')?"; secure; HttpOnly":"; HttpOnly");
     ses->cookie = zx_alloc_sprintf(cf->ctx, 0, "$Version=1; %s=%s",
 				   cf->ses_cookie_name, ses->sid);
   }
