@@ -473,7 +473,7 @@ void zxid_wsf_sign(zxid_conf* cf, int sign_flags, struct zx_wsse_Security_s* sec
     ASSERTOPI(ZXID_N_WSF_SIGNED_HEADERS, >=, n_refs);
 
     if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert wsc")) {
-      sec->Signature = zxsig_sign(cf->ctx, n_refs, refs, sign_cert, sign_pkey);
+      sec->Signature = zxsig_sign(cf->ctx, n_refs, refs, sign_cert, sign_pkey, cf->xmldsig_sig_meth, cf->xmldsig_digest_algo);
       zx_add_kid(&sec->gg, &sec->Signature->gg);
     }
   }
