@@ -1,5 +1,5 @@
 /* zxpasswd.c  -  Password creation and user management tool
- * Copyright (c) 2012 Synergetics SA (sampo@synergetics.be), All Rights Reserved.
+ * Copyright (c) 2012-2015 Synergetics SA (sampo@synergetics.be), All Rights Reserved.
  * Copyright (c) 2009-2011 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.
  * This is confidential unpublished proprietary source code of the author.
  * NO WARRANTY, not even implied warranties. Contains trade secrets.
@@ -13,6 +13,7 @@
  * 1.2.2011,   tweaked -at option --Sampo
  * 5.2.2012,   changed -c flag to -n to reserve -c for config (to be consistent with other utils) --Sampo
  * 24.4.2012,  obsoleted PATH=/var/zxid/idp. From now on, just use /var/zxid/ or VPATH --Sampo
+ * 29.5.2015,  added generation of PINs --Sampo
  *
  * See also: http://www.users.zetnet.co.uk/hopwood/crypto/scan/ph.html
  * http://www.usenix.org/events/usenix99/provos/provos_html/index.html
@@ -51,7 +52,7 @@
 
 char* help =
 "zxpasswd  -  Password creation and user management tool R" ZXID_REL "\n\
-Copyright (c) 2012 Synergetics SA (sampo@synergetics.be), All Rights Reserved.\n\
+Copyright (c) 2012-2015 Synergetics SA (sampo@synergetics.be), All Rights Reserved.\n\
 Copyright (c) 2009-2011 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.\n\
 NO WARRANTY, not even implied warranties. Licensed under Apache License v2.0\n\
 See http://www.apache.org/licenses/LICENSE-2.0\n\
@@ -68,6 +69,7 @@ Usage: zxpasswd [options] user [udir] <passwd      # Set user's password\n\
   -a               Authenticate as user. exit(2) value 0 means success\n\
   -l               List user info. If no user is specified, lists all users.\n\
   -t N             Choose password hash type: 0=plain, 1=MD5 (default), y=yubikey\n\
+  -p               The password is considered as PIN and written in .pin\n\
   -basic uid:pw    Print HTTP Basic Authenication blob, i.e. URI encoded base64 over uid:pw\n\
   -v               Verbose messages.\n\
   -q               Be extra quiet.\n\
@@ -83,6 +85,7 @@ int verbose = 1;
 int create = 0;
 int an = 0;
 int list = 0;
+int is_pin = 0;
 char* hash_type = "1";
 char* udir = UDIR;
 char* user = 0;
@@ -201,6 +204,14 @@ static void opt(int* argc, char*** argv, char*** env)
       }
       break;
 
+    case 'p':
+      switch ((*argv)[0][2]) {
+      case '\0':
+	is_pin = 1;
+	continue;
+      }
+      break;
+
     case 'v':
       switch ((*argv)[0][2]) {
       case '\0':
@@ -238,7 +249,7 @@ static int list_user(char* userdir, char* udir)
   DIR* dir;
   dir = opendir(userdir);
   if (!dir) {
-    perror("opendir for /var/zxid/idpuid/USER userdir (or other if configured)");
+    perror("opendir for /var/zxid/uid/USER userdir (or other if configured)");
     D("failed path(%s)", userdir);
     return 4;
   }
@@ -394,7 +405,7 @@ int main(int argc, char** argv, char** env)
   if (argc)
     user = argv[0];
   else if (!list) {
-    fprintf(stderr, "Too few arguments (%d). Specify at least user name.\n%s", argc, help);
+    fprintf(stderr, "Too few arguments (%d). Specify at least username.\n%s", argc, help);
     /*fprintf(stderr, "version=0x%06x rel(%s)\n", zxid_version(), zxid_version_str());*/
     exit(3);
   }
@@ -564,7 +575,7 @@ Personalizing yubikeys 2009: get and compile libyubikey-1.5 and ykpers-1.0
 ykpersonalize -y -v -ofixed=refucenikj -a6012cad434c66ab87d43d4babe463231
 ykdebug 6012cad434c66ab87d43d4babe463231 refucenikjdbrgulutnjhurchlkcckdkergfitcebf
 
-Here -ofixed specifies the "username" for purposes of /var/zxid/idpuid and
+Here -ofixed specifies the "username" for purposes of /var/zxid/uid and
 -a specifies the AES128 key that will be put in .pw file as follows:
 
 y5e0w.RTowQpk
